import java.nio.channels.*;
import java.util.*;
import java.util.regex.*;

/**
 * Classe di base per la creazione di un agente basato sull'infrastruttura Quagent, implementa
 * le funzioni basilari di comunicazione con l'infrastruttura.
 */
public class BasicQuagent {

    protected LinkedList event_list=new LinkedList();
    private QuagentSocket socket;
    private static Pattern pattern;
    
    /**
     * Costruttore della classe.
     * @param s Oggetto SocketChannel contenente i parametri di connessione al canale di comunicazione.
     */
    public BasicQuagent (SocketChannel s) {
    	socket = new QuagentSocket(s);
    	pattern = Pattern.compile("\\(([^\\)]+)\\)");
    }

    /**
     * Invia una richiesta sul canale di comunicazione.
     * @param req Richiesta da inviare sul canale. Notare che Request è una classe astratta, da specializzare per ogni comando.
     * @return Lo stesso oggetto passato come parametro.
     */
    public Request send(Request req) {
    	// don't forget the trailing \n
    	System.out.println("Sending: "+req);
    	socket.sendString(req.toString()+"\n");
    	return req;
    }

    /**
     * Legge il messaggio di benvenuto da parte dell'infrastruttura.
     */
    public void getGreeting() {
    	socket.getLine();
    }

    /**
     * Legge dal canale di comunicazione la risposta alla richiesta passata come parametro e già
     * inviata all'infrastruttura,
     * @param request Richiesta inviata sul canale di cui si attende la risposta.
     * @return La risposta alla richiesta passata come argomento.
     */
    public Response getResponse(Request request) {
		String received = null;
		Response response = null;
		String request_type = null;
		boolean matched = false;
	
		if(request == null)
		    matched = true;
		do {
			// wait for a response to come
			do {
				received = socket.receiveString();
			} while (received.length() < 1);
			
			// possibly multiple responses in the socket
			// break into individual messages
			String[] message = received.split("\\n");
			for(int i=0; i<message.length; ++i) {
				System.out.println("Received msg: "+i+" " +message[i]);
				// retrieve and then remove the echoing part
				Matcher matcher = pattern.matcher(message[i]);
				String msg = matcher.replaceFirst("");
				String[] tokens = msg.split("\\s+");
				// keep the echo'ed part
				if(!msg.equals(message[i]) ) {
					request_type = matcher.group(1);
				}
				if(tokens[0].equals("TELL")) { // it's an event
					event_list.add(new QuagentEvent(tokens));
				} else {
					// if not "TELL", request_type must not be null
					// check if the response it what we expected
					if(request_type.equals(request.toString()))
						matched = true;
					// create specific response according to the request
					if (request_type.indexOf("ASK RADIUS") != -1)
						response = new RadiusResponse();
					else if(request_type.indexOf("DO GETWELLBEING") != -1)
						response = new WellbeingResponse();
					else if(request_type.indexOf("ASK RAYS") != -1)
						response = new RaysResponse();
					else
						response = new Response();
					if(tokens[0].equals("OK")) {
						response.isOK(true);
						response.setValue(tokens);
					}else {
						response.isOK(false);
					}
				}
			}
		}while (!matched);
		return response;
    }

    public void testResponse() {
		String received = null;			
		received = socket.receiveString();
		if(received.length()>=1) {
			
			// possibly multiple responses in the socket
			// break into individual messages
			String[] message = received.split("\\n");
			for(int i=0; i<message.length; ++i) {
				System.out.println("Received msg: "+i+" " +message[i]);
				// retrieve and then remove the echoing part
				Matcher matcher = pattern.matcher(message[i]);
				String msg = matcher.replaceFirst("");
				String[] tokens = msg.split("\\s+");
				if(tokens[0].equals("TELL")) { // it's an event
					event_list.add(new QuagentEvent(tokens));
				} 
			}
		}
    }
    
    /**
     * Avvia l'agente Quagent. L'implementazione attuale si limita a stampare una stringa a video, la funzione deve
     * essere implementata nella classe che estende BasicQuagent.
     */
    public void run() {
    	System.out.println("BasicQuagent's running.");
    }

    /**
     * Verifica la presenza di un evento non ancora gestito e procede alla sua gestione.  L'implementazione attuale si 
     * limita a stampare una stringa a video, la funzione deve essere implementata nella classe che estende BasicQuagent.
     */
    public void handleEvent() {
    	System.out.println("Event's coming.");
    }

}

