import java.nio.channels.*;
import javax.swing.*;
import alice.tuprolog.*;
import java.io.*;

/**
 * Classe di agenti di tipo Guard, estende la classe base BasicQuagent implementandone i metodi run() e handleEvent().
 * @author Francesco Berardi
 */
public class GuardAgent extends BasicQuagent {
	GuardMain msgArea;
	private boolean mission_completed = false;
	
	/**
	 * Costruttore della classe.
	 * @param s Oggetto di tipo SocketChannel contenente i parametri di connessione al canale di comunicazione.
	 * @param msg TextArea che visualizzerà i messaggi dell'agente.
	 */
	public GuardAgent(SocketChannel s, GuardMain  msg) {
		super(s);
		msgArea=msg;
	}
	
    /**
     * Avvia l'agente Quagent.
     */
    public void run() {
    	SolveInfo sol;
    	Theory t;
    	Prolog engine=new Prolog();
    	String next_cmd="", next_par="", tell_state="", agent_state="";
        String cmd_story="[]", cmd_response="[]", theory_file="";
    	QuagentEvent ev;
    	Request cmd;
    	Response cmd_rsp;
    	
    	msgArea.appendMsg(">> Agente avviato.\n");
    	// Carica la teoria prolog di gestione        
    	theory_file="guardA.pl";
        try {
        	 t=new Theory(new FileInputStream(theory_file));
             engine.setTheory(t);
        } catch(InvalidTheoryException e1) {
        	System.out.println(">> Errore nel caricamento della teoria prolog. (1)\n");
			return;
        } catch(IOException e2) {
            System.out.println(">> Errore nel caricamento della teoria prolog. (2)\n");
			return;
        }
        
    	// Gestione del comportamento dell'agente sino al completamendo della missione
        getGreeting();
        
        agent_state="pattuglia";
        cmd_story="[start]";
    	tell_state="false";
    	
        while(!mission_completed) {
        	// Test per il verificarsi di un evento TELL
        	while(!event_list.isEmpty()){
        	    ev = (QuagentEvent)event_list.removeFirst();
        	    if(ev.type().compareTo("DEAD")==0) 
        	    	mission_completed = true;
        	    if(ev.type().compareTo("TELL")==0) 
        	    	tell_state="true";
        	    msgArea.appendMsg(">> "+ev.type()+" "+ev.value()+"\n");
        	}
        	
        	// Calcolo del comando da eseguire
        	try {
        		sol=engine.solve("nextCmd( "+cmd_story+","+tell_state+","+cmd_response+","+agent_state+",CMD,PAR,STOR,STATE).");
        	 } catch(Exception ex){
        		 System.out.println(">> Errore nella risoluzione del goal. (3)\n");
                 return;               
            }
        	//System.out.println(sol.toString());
        	if(sol.isSuccess()) {	
            	try {
                    next_cmd = new String(sol.getTerm("CMD").toString());
                    next_par = new String(sol.getTerm("PAR").toString());
                    cmd_story = new String(sol.getTerm("STOR").toString());
                    agent_state = new String(sol.getTerm("STATE").toString());
               } catch(Exception ex){
            	  	System.out.println(">> Errore nella risoluzione del goal. (4)\n"); 
                    return;               
               }
            }
        	// Crea l'oggetto richiesta corrispondente al comando calcolato 
        	if(next_cmd.compareTo("turn")==0)  {
        		cmd=new TurnCommand(java.lang.Float.valueOf(next_par).floatValue());
        		msgArea.appendMsg(">> "+cmd.toString());
        		send(cmd);
        		cmd_rsp = getResponse(cmd);
        		if(cmd_rsp.isOK()) msgArea.appendMsg(" - OK\n");
            	cmd_response="[]";
        	} else if(next_cmd.compareTo("walk")==0)  {
        		cmd=new WalkCommand(java.lang.Float.valueOf(next_par).floatValue());
        		msgArea.appendMsg(">> "+cmd.toString());
        		send(cmd);
        		cmd_rsp = getResponse(cmd);
        		if(cmd_rsp.isOK()) msgArea.appendMsg(" - OK\n");
            	cmd_response="[]";
        	} else if(next_cmd.compareTo("stand")==0)  {
        		cmd=new StandCommand();
        		msgArea.appendMsg(">> "+cmd.toString());
        		send(cmd);
        		cmd_rsp = getResponse(cmd);
        		if(cmd_rsp.isOK()) msgArea.appendMsg(" - OK\n");;
        		cmd_response="[]";
        	} else if(next_cmd.compareTo("radius")==0)  {
        		cmd=new RadiusQuery(java.lang.Float.valueOf(next_par).floatValue());
        		msgArea.appendMsg(">> "+cmd.toString());
        		send(cmd);
        		cmd_rsp = getResponse(cmd);
        		if(cmd_rsp.isOK()) msgArea.appendMsg(" - "+((RadiusResponse)cmd_rsp).dumpList()+"\n");
        		cmd_response=((RadiusResponse)cmd_rsp).dumpList();
        	} else if(next_cmd.compareTo("rays")==0)  {
        		cmd=new RaysQuery(java.lang.Integer.valueOf(next_par).intValue());
        		msgArea.appendMsg(">> "+cmd.toString());
        		send(cmd);
        		cmd_rsp = getResponse(cmd);
        		if(cmd_rsp.isOK()) msgArea.appendMsg(" - "+((RaysResponse)cmd_rsp).dumpList()+"\n");
        		cmd_response=((RaysResponse)cmd_rsp).dumpList();       		
        	} else if(next_cmd.compareTo("victory")==0)  {
        		mission_completed=true;  
        		msgArea.appendMsg("-------------------------------------------------\n");
        		msgArea.appendMsg("--   LA GUARDIA VINCE LA PARTITA   --\n");
        		msgArea.appendMsg("-------------------------------------------------\n");
        		 JOptionPane.showMessageDialog(null, "LA GUARDIA VINCE LA PARTITA", "GUARD", JOptionPane.INFORMATION_MESSAGE);
        	}
        	testResponse();
    		tell_state="false";
    		//try{
    		//	Thread.sleep(500);
    		//}catch (InterruptedException e) { }
    		for(int ii=0; ii<1000000000; ii++);
    	}
    }

    /**
     * Verifica la presenza di un evento non ancora gestito e procede alla sua gestione.
     */
    public void handleEvent() {
    	while(!event_list.isEmpty()){
    	    QuagentEvent ev = (QuagentEvent)event_list.removeFirst();
    	    if(ev.type() == "DEAD") mission_completed = true;
    	    msgArea.appendMsg(">> "+ev.type()+" "+ev.value()+"\n");
    	}	
    }
    
}


