import java.net.*;
import java.io.*;
import java.nio.channels.*;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.JPanel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Classe di avvio dell'applicazione, visualizza la finestra dei messaggi ed avvia l'agente Guard.
 * @author Francesco Berardi
 */
public class GuardMain implements ActionListener {

    private JFrame msgFrame;
    private JPanel msgPanel;
    private JTextArea msgArea;
    private JScrollBar scrollbar;
    SocketChannel sChannel;
	
    /**
	 * Costruttore della classe.
	 */
	public GuardMain() {
		super();
		System.out.println("Agente Guard avviato.");
		// Crea la finestra dei messaggi
		initMsgFrame();
		// Associa un gestore di eventi alla finestra
        msgFrame.addWindowListener(new WindowAdapter() {
        	public void windowClosing(WindowEvent e) {
        		quit();
            }});
        try{
        	// Apre il canale di comunicazione
        	sChannel = SocketChannel.open();
        	sChannel.configureBlocking(false);
        	sChannel.connect(new InetSocketAddress("localhost", 33333));
        	while (!sChannel.finishConnect()) {
        	}
        	appendMsg(">> Canale di comunicazione aperto con l'host [localhost:33333]\n");
        	// Crea ed avvia l'agente Guard
        	BasicQuagent agent = new GuardAgent(sChannel, this);
        	agent.run();
        	// Al termine dell'esecuzione arresta l'applicazione
        	quit();
        } catch (UnknownHostException e1) {
        	System.out.println("UnknownHostException:" + e1.getMessage());
        } catch (ConnectException e2){
        	System.out.println("ConnectException:" + e2.getMessage());
        } catch (IOException e3){
        	System.out.println("IO Error:" + e3.getMessage());
        }	      
	}
	
	/**
	 * Punto di ingresso dell'applicazione.
	 * @param args Parametri della linea di comando.
	 */
	public static void main(String[] args) {
		new GuardMain();
	}

	/**
	 * Inizializza la finestra dei messaggi.
	 */
	private void initMsgFrame() {
		// Crea gli oggetti grafici
		msgFrame = new JFrame("GUARD - Log dei messaggi");
		msgPanel = new JPanel();
		msgArea = new JTextArea(10,60);
		msgArea.setEditable(false);
		JScrollPane scroller = new JScrollPane(msgArea);
		scrollbar = scroller.getVerticalScrollBar();
		// Aggiunge gli oggetti grafici al frame
		msgPanel.add(scroller, BorderLayout.CENTER); 
		msgFrame.getContentPane().add(msgPanel, BorderLayout.NORTH);
		msgFrame.pack();
		msgFrame.setVisible(true);
	}
	
	/**
	 * Chiude correttamente l'applicazione, deallocando gli oggetti grafici e chiudendo
	 * il collegamento con l'infrastruttura.
	 */
    private void quit(){
    	System.out.println("Arresto dell'agente Thief...");
    	// Chiude il socket
    	try{
    		sChannel.socket().close();
    		sChannel.close();
        } catch (UnknownHostException e1) {
        	System.out.println("UnknownHostException:" + e1.getMessage());
        } catch (ConnectException e2){
        	System.out.println("ConnectException:" + e2.getMessage());
        } catch (IOException e3){
        	System.out.println("IO Error:" + e3.getMessage());
        }
	    // Elimina la finestra grafica 
    	msgFrame.setVisible(false);
    	msgFrame.dispose();
    	System.out.println("Agente Guard arrestato.");
    	// Arresta l'applicazione
    	System.exit(0);
    }
    
    public void actionPerformed(ActionEvent e) {
        System.err.println("actionPerformed()");
    }
    
    /**
     * Aggiunge un messaggio alla finestra dei messaggi.
     * @param msg Stringa contenente il messaggio da aggiungere.
     */
    public void appendMsg(String msg) {
    	msgArea.append(msg);
    	scrollbar.setValue(scrollbar.getMaximum());
    }
}
