% COMANDI DISPONIBILI: 	TURN, WALK, STAND, RADIUS, RAYS
% STATI DISPONIBILI: 		PATTUGLIA, CATTURA


% nextCmd(+PrevCmdStory,+TellState,+CmdResponse,+PrevAgentState,
%	-NextCmd,-NextPar,-NextCmdStory,-NextAgentState)
%		PrevCmdStory: storico attuale dei comandi eseguiti
%		TellState: stato attuale della risposta TELL
%		CmdResponse: lista contenente la risposta all'ultimo comando eseguito
%		PrevAgentState: stato attuale dell'agente
% 		NextCmd: prossimo comando da eseguire
%		NextPar: parametro del comando da eseguire (0 se non applicabile)
%		NextCmdStory: storico aggiornato dei comandi eseguiti
%		NextAgentState: nuovo stato dell'agente dopo l'esecuzione del comando


nextCmd([start],_,_,STATE,CMD,PAR,STOR,STATE):- CMD is turn, PAR is 90, STOR is [turn].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MODALITA' PATTUGLIA

% Dopo aver ruotato cammina in avanti
nextCmd([turn|PSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is pattuglia, 
	union([turn],PSTOR,STOR1), union([walk],STOR1,STOR).

% Dopo aver camminato due volte ruota a destra
nextCmd([walk|TSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- countPreviousWalk(TSTOR,0,COUNT,_), COUNT==0,
	CMD is walk, PAR is 50, STATE is pattuglia, union([walk],TSTOR,STOR1), union([walk],STOR1,STOR).
nextCmd([walk|TSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- countPreviousWalk(TSTOR,0,COUNT,[turn|TAIL]), COUNT==1,
	CMD is radius, PAR is 100, STATE is pattuglia, union([walk],TSTOR,STOR1),union([radius],STOR1,STOR).
nextCmd([walk|TSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- countPreviousWalk(TSTOR,0,COUNT,[radius|TAIL]), COUNT==1,
	CMD is turn, PAR is -90, STATE is pattuglia, union([walk],TSTOR,STOR1),union([turn],STOR1,STOR).
nextCmd([walk|TSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- countPreviousWalk(TSTOR,0,COUNT,[stand|TAIL]), COUNT==1,
	CMD is radius, PAR is 100, STATE is pattuglia, union([walk],TSTOR,STOR1),union([radius],STOR1,STOR).
nextCmd([walk|TSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- countPreviousWalk(TSTOR,0,COUNT,[rays|TAIL]), COUNT==1,
	CMD is radius, PAR is 100, STATE is pattuglia, union([walk],TSTOR,STOR1),union([radius],STOR1,STOR).

% Dopo un comando di stand o rays riprende a camminare
nextCmd([stand|PSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is pattuglia,
	union([stand],PSTOR,STOR1), union([walk],STOR1,STOR).
nextCmd([rays|PSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is pattuglia,
	union([rays],PSTOR,STOR1), union([walk],STOR1,STOR).

% Nessun oggetto in vista: continua a camminare normalmente	
nextCmd([radius|PSTOR],_,[],pattuglia,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is pattuglia,
	union([radius],PSTOR,STOR1), union([walk],STOR1,STOR).
% Ladro in vista: passa nello stato di cattura
nextCmd([radius|PSTOR],_,RESP,pattuglia,CMD,PAR,STOR,STATE):-
	findall(X,member(obj(randal,X,_,_),RESP,_),L), length(L,N), N==1, 
	getDistance(RESP,D), D@>50, getAngle(RESP,A), CMD is turn, PAR is A, STATE is cattura, 
	union([radius], PSTOR,STOR1), union([turn],STOR1,STOR).
% Ladro a distanza di cattura: la guardia vince
nextCmd([radius|PSTOR],_,RESP,pattuglia,CMD,PAR,STOR,STATE):-
	findall(X,member(obj(randal,X,_,_),RESP,_),L), length(L,N), N==1, 
	getDistance(RESP,D), D@=<50, CMD is victory, PAR is 0, STATE is cattura, 
	union([radius], PSTOR,STOR1), union([victory],STOR1,STOR).
% Oggetto generico in vista: continua a camminare normalmente
nextCmd([radius|PSTOR],_,_,pattuglia,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is pattuglia,
	union([radius],PSTOR,STOR1), union([walk],STOR1,STOR).	
		
% countPreviousWalk(+CmdStory,+CountBase,-Count,-Tail)
%		CmdStory: storico dei comandi su cui eseguire la verifica
%		CountBase: valore iniziale da cui iniziare a contare (normalmente 0)
%		Count: numero di occorrenze consecutive del comando WALK a partire dalla testa dello storico
%		Tail: parte restante dello storico dei comandi dopo la rimozione dei comandi WALK conteggiati

countPreviousWalk([turn|TAIL],COUNT,COUNT,TAIL1):-union([turn],TAIL,TAIL1).
countPreviousWalk([stand|TAIL],COUNT,COUNT,TAIL1):-union([stand],TAIL,TAIL1).
countPreviousWalk([radius|TAIL],COUNT,COUNT,TAIL1):-union([radius],TAIL,TAIL1).
countPreviousWalk([rays|TAIL],COUNT,COUNT,TAIL1):-union([rays],TAIL,TAIL1).
countPreviousWalk([walk|TAIL],PCOUNT,COUNT,TAIL1):-countPreviousWalk(TAIL,PCOUNT,PCOUNT1,TAIL1),COUNT is PCOUNT1+1.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MODALITA' CATTURA

% Dopo una rotazione cammina in avanti	
nextCmd([turn|PSTOR],_,_,cattura,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 50, STATE is cattura, 
	union([turn],PSTOR,STOR1), union([walk],STOR1,STOR).

% Dopo un comando di stand o rays controlla la presenza del ladro
nextCmd([stand|PSTOR],_,_,cattura,CMD,PAR,STOR,STATE):- CMD is radius, PAR is 100, STATE is cattura,
	union([stand],PSTOR,STOR1), union([radius],STOR1,STOR).
nextCmd([rays|PSTOR],_,_,cattura,CMD,PAR,STOR,STATE):- CMD is radius, PAR is 100, STATE is cattura,
	union([rays],PSTOR,STOR1), union([radius],STOR1,STOR).

% Dopo aver camminato controlla la presenza del ladro
nextCmd([walk|TSTOR],_,_,cattura,CMD,PAR,STOR,STATE):- CMD is radius, PAR is 100, STATE is cattura, 
	union([walk],TSTOR,STOR1), union([radius],STOR1,STOR).
	
% Nessun oggetto in vista:attendi
nextCmd([radius|PSTOR],_,[],cattura,CMD,PAR,STOR,STATE):- CMD is stand, PAR is 0, STATE is cattura,
	union([radius],PSTOR,STOR1), union([stand],STOR1,STOR).
% Ladro in vista: continua l'avvicinamento
nextCmd([radius|PSTOR],_,RESP,cattura,CMD,PAR,STOR,STATE):-
	findall(X,member(obj(randal,X,_,_),RESP,_),L), length(L,N), N==1, 
	getDistance(RESP,D), D@>50, getAngle(RESP,A), CMD is turn, PAR is A, STATE is cattura, 
	union([radius], PSTOR,STOR1), union([turn],STOR1,STOR).
% Ladro a distanza di cattura: la guardia vince
nextCmd([radius|PSTOR],_,RESP,cattura,CMD,PAR,STOR,STATE):-
	findall(X,member(obj(randal,X,_,_),RESP,_),L), length(L,N), N==1, 
	getDistance(RESP,D), D@=<50, CMD is victory, PAR is 0, STATE is cattura, 
	union([radius], PSTOR,STOR1), union([victory],STOR1,STOR).
% Oggetto generico in vista: attendi
nextCmd([radius|PSTOR],_,_,cattura,CMD,PAR,STOR,STATE):- CMD is stand, PAR is 0, STATE is cattura,
	union([radius],PSTOR,STOR1), union([stand],STOR1,STOR).

% Legge le coordinate relative del ladro e ne calcola la distanza
getDistance(L,D):- member(obj(randal,X,Y,_),L,_), D is sqrt((X*X)+(Y*Y)).

% Legge le coordinate relative del ladro e calcola l'angolo di rotazione
getAngle(L,A):- member(obj(randal,X,Y,_),L,_), A1 is (atan(Y/X)*180)/3.1415926535, corrAng(A1,X,Y,A).

% Corregge l'angolo A passato come argomento in base al quadrante in cui ci si trova
corrAng(A,X,Y,A1):- Y==0, X@>0, A1 is 0. 
corrAng(A,X,Y,A1):- Y==0, X@<0, A1 is 3.1415926535.
corrAng(A,X,Y,A1):- X==0, Y@>0, A1 is (3.1415926535/2).
corrAng(A,X,Y,A1):- X==0, Y@<0, A1 is 3*(3.1415926535/2).
corrAng(A,X,Y,A1):-  X@>=0, Y@>=0, A1 is A.
corrAng(A,X,Y,A1):-  X@>=0, Y@<0, A1 is (A+(2*3.1415926535)).
corrAng(A,X,Y,A1):-  X@<0, Y@>=0, A1 is (A+3.1415926535).
corrAng(A,X,Y,A1):-  X@<0, Y@<0, A1 is (A+3.1415926535).

			
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNZIONI GENERALI

% union(+M1,+M2,-Mout), aggiunge M2 in coda ad M1
union([],M1,M1).
union([M|Ms],M1,[M|Mo]):-union(Ms,M1,Mo).

% member(?E,?M,?O), trova tutte le occorrenze di E in M e restituisce il resto in o
member(E,[E|T],T).
member(E,[H|T],[H|R]):-member(E,T,R).