
//Wumpus World interface, made for Jess
//Pete Barnum pbarnum@cs.rochester.edu

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class wumpus extends JPanel implements ActionListener {
    
    public static final int SIZEX = 5;
    public static final int SIZEY = 5;
    public static final int DEBUG = 2;
    
    ImageIcon IemptySquare, IemptySquare2, Iwumpus, Istench, Igold, Ipit, Ibreeze, Istart, Iplayer;
    
    int dungeon[][][]=new int[SIZEX][SIZEY][4]; //visible,stench,pit,breeze
    int playerX = 0, playerY=0;
    int wumpusX = 2, wumpusY = 2;
    int goldX = 1, goldY = 3;
    boolean playerAlive, hasGold, wumpusAlive;
    
    //For testing purposes
    public static void main( String args[] )
    {
	//wumpus board = new wumpus("images/");
	wumpus board = new wumpus("images/");
	
	JFrame window = new JFrame( "Wumpus World" );
	window.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
	
	Container container = window.getContentPane();
	container.add( board );
	
	window.pack();
	window.setSize(500,500);
	window.setVisible( true );
    }
    
    
    public wumpus(String imagePath){
	
	int i,j,k,l;
	int offsetX, offsetY;
	
	IemptySquare = new ImageIcon(getClass().getResource(imagePath + "emptySquare.gif"));
	IemptySquare2 = new ImageIcon(getClass().getResource(imagePath + "emptySquare2.gif"));
	Iwumpus = new ImageIcon(getClass().getResource(imagePath + "wumpus.gif"));
	Istench = new ImageIcon(getClass().getResource(imagePath + "stench.gif"));
	Igold = new ImageIcon(getClass().getResource(imagePath + "gold.gif"));
	Ipit = new ImageIcon(getClass().getResource(imagePath + "pit.gif"));
	Ibreeze = new ImageIcon(getClass().getResource(imagePath + "breeze.gif"));
	Istart = new ImageIcon(getClass().getResource(imagePath + "start.gif"));
	Iplayer = new ImageIcon(getClass().getResource(imagePath + "player.gif"));

	playerAlive=true;
	hasGold=false;
	wumpusAlive=true;

	updateCircumstance();

	//Initialize any pits here
	dungeon[4][4][2]=1;
	
	//make breeze
	for(i=0;i<SIZEX;i++)
	    for(j=0;j<SIZEY;j++){
		if(dungeon[i][j][2]==1){
		    offsetX=i+1; offsetY=j;
		    if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
			dungeon[offsetX][offsetY][3]=1;
		    }
		    offsetX=i-1; offsetY=j;
		    if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
			dungeon[offsetX][offsetY][3]=1;
		    }
		    offsetX=i; offsetY=j+1;
		    if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
			dungeon[offsetX][offsetY][3]=1;
		    }
		    offsetX=i; offsetY=j-1;
		    if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
			dungeon[offsetX][offsetY][3]=1;		
			
		    }
		}
	    }
	
    }
    
    public void paintComponent(Graphics g){
	
	super.paintComponent(g);
	
	int i,j;

	//Display dungeon
	for(i=0;i<SIZEX;i++){
	    for(j=0;j<SIZEY;j++){
		if(dungeon[i][j][0]==1){
		    IemptySquare.paintIcon(this,g,i*75,j*75);
		    
		    if(dungeon[i][j][1]==1) Istench.paintIcon(this,g,i*75+5,j*75+1);
		    if(dungeon[i][j][2]==1) Ipit.paintIcon(this,g,i*75+10,j*75+10);
		    if(dungeon[i][j][3]==1) Ibreeze.paintIcon(this,g,i*75+5,j*75+23);
		}
		else if(DEBUG>0){
		    IemptySquare2.paintIcon(this,g,i*75,j*75);
		    if(dungeon[i][j][1]==1) Istench.paintIcon(this,g,i*75+5,j*75+1);
		    if(dungeon[i][j][2]==1) Ipit.paintIcon(this,g,i*75+10,j*75+10);
		    if(dungeon[i][j][3]==1) Ibreeze.paintIcon(this,g,i*75+5,j*75+23);
		}
		else{
		    IemptySquare2.paintIcon(this,g,i*75,j*75);
		}
	    }
	}
	
	//show wumpus and gold anyway, if debugging
	if(DEBUG>0){
	    if(!hasGold) Igold.paintIcon(this,g,goldX*75+5,goldY*75+45);
	    if(wumpusAlive)	Iwumpus.paintIcon(this,g,wumpusX*75+10,wumpusY*75+10);
	}
	
	//Display gold and player
	if(dungeon[goldX][goldY][0]==1 && !hasGold) Igold.paintIcon(this,g,goldX*75+5,goldY*75+45);
	Iplayer.paintIcon(this,g,playerX*75+10,playerY*75+10);
	
	
	
    }
    
    public boolean move_left(){
	if(DEBUG>1){
	    badWait(1000);
	}

	if(playerX>0 && playerAlive) {
	    playerX--;
	    updateCircumstance();
	    return true;
	}
	else
	    return false;
    }
    
    public boolean move_right(){
	if(DEBUG>1){
	    badWait(1000);
	}

	if(playerX<SIZEX-1 && playerAlive) {
	    playerX++;
	    updateCircumstance();
	    return true;
	}
	else
	    return false;
    }
    
    public boolean move_up(){
	if(DEBUG>1){
	    badWait(1000);
	}

	if(playerY>0 && playerAlive) {
	    playerY--;
	    updateCircumstance();
	    return true;
	}
	else
	    return false;
    }

    public boolean move_down(){
	if(DEBUG>1){
	    badWait(1000);
	}

	if(playerY<SIZEY-1 && playerAlive) {
	    playerY++;
	    updateCircumstance();
	    return true;
	}
	else
	    return false;
    }
    
    //Interface for Jess quesioning the world
    public boolean hasStench(){
	if(dungeon[playerX][playerY][1]==1) return true;
	return false;
    }
    public boolean hasBreeze(){
	if(dungeon[playerX][playerY][3]==1) return true;
	return false;
    }
    public boolean hasGlitter(){
	if(playerX==goldX && playerY==goldY) return true;
	return false;
    }
    public int getX(){
	return playerX;
    }
    public int getY(){
	return playerY;
    }

    //update the world and player's knowledge of the world
    void updateCircumstance(){
	
	int i,j;
	int offsetX, offsetY;

	//Update player visibility, for 1x1 view
	dungeon[playerX][playerY][0]=1;			       

	//make stench, if wumpus is alive
	offsetX=wumpusX+1; offsetY=wumpusY;
	if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
	    if(wumpusAlive) dungeon[offsetX][offsetY][1]=1;
	    else dungeon[offsetX][offsetY][1]=0;
	}
	offsetX=wumpusX-1; offsetY=wumpusY;
	if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
	    if(wumpusAlive) dungeon[offsetX][offsetY][1]=1;
	    else dungeon[offsetX][offsetY][1]=0;
	}
	offsetX=wumpusX; offsetY=wumpusY+1;
	if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
	    if(wumpusAlive) dungeon[offsetX][offsetY][1]=1;
	    else dungeon[offsetX][offsetY][1]=0;
	}
	offsetX=wumpusX; offsetY=wumpusY-1;
	if(offsetX>=0 && offsetY >= 0 && offsetX < (SIZEX) && offsetY < (SIZEY)){
	    if(wumpusAlive) dungeon[offsetX][offsetY][1]=1;
	    else dungeon[offsetX][offsetY][1]=0;
	}
	
	//if the player died or found gold, update
	if(dungeon[playerX][playerY][2]==1) playerAlive=false;
	if(wumpusX==playerX && wumpusY==playerY) playerAlive=false;
	if(goldX==playerX && goldY==playerY) hasGold=true;
	
	repaint();
    }
    
    public void actionPerformed( ActionEvent actionEvent ){
	repaint();
    }
    
    //Ugly wait function, to simulate something else running with Jess program
    public void badWait(long howLong){
	int i=0,j=0;
	for(i=0;i<howLong*100000;i++){
	    j++;
	}
    }
    
}//end class wumpus

