%{
#include "quagent.h"

//#define TEST

int yylex(void);
void yyerror(const char *err) {printf("%s\n", err);}

#ifdef TEST
int config_num_quagents;
#else
extern int config_num_quagents;
extern quagent_spawn_info quagent_spawns[];
extern edict_t *fpsclient;
#endif

static void spawn_item(int, float, float, float);
static void set_quagent_type(int type) ;
static void set_quagent_state(int state, float val);
static void set_quagent_position(float, float, float);
%}

%union {
	int index;
	float value;
	char *name;
}
%token <value> NUMBER
%token <index> BOX TOFU BATTERY GOLD DATA KRYPTONITE HEAD
%token <index> LIFETIME INITIALWISDOM INITIALENERGY INITIALWEALTH INITIALHEALTH 
%token <index> ENERGYLOWTHRESHOLD AGEHIGHTHRESHOLD
%token <index> BERSERK CHICK GLADIATOR GUNNER INFANTRY PARASITE SOLDIER
%token <index> MUTANT FLOAT
%token QUAGENT QUAGENT_TYPE QUAGENT_POSITION
%type <index> item_name quagent_type quagent_state
%%

config_entries: 
		| config_entries config_entry
		;

config_entry: item_entry | quagent_entry
		;

item_entry: item_name NUMBER NUMBER NUMBER { spawn_item($1, $2, $3, $4); };
				
item_name: BOX|TOFU|BATTERY|GOLD|DATA|KRYPTONITE|HEAD ;

quagent_entry: QUAGENT '{' quagent_property_list '}' { 
	if(config_num_quagents>=MAX_CONFIG_QUAGENTS) 
	    YYABORT; 
	else 
	    config_num_quagents ++; }
		;

quagent_property_list: 
			| quagent_property_list quagent_property
			;

quagent_property: QUAGENT_TYPE quagent_type { set_quagent_type($2); }
			| quagent_state NUMBER { set_quagent_state($1, $2); } 
			| QUAGENT_POSITION NUMBER NUMBER NUMBER {set_quagent_position($2, $3, $4);}
			;
			
quagent_type:BERSERK|CHICK|GLADIATOR|GUNNER|INFANTRY|PARASITE|SOLDIER|MUTANT|FLOAT
		;

quagent_state:LIFETIME|INITIALWISDOM|INITIALENERGY|INITIALWEALTH|INITIALHEALTH|ENERGYLOWTHRESHOLD|AGEHIGHTHRESHOLD
		;

%%

void spawn_item(int type, float x, float y, float z)
{
#ifdef TEST
    printf("spawn item: %d at %f %f %f\n", type, x, y, z);
#else
    fpsclient->s.origin[0] = x;
    fpsclient->s.origin[1] = y;
    fpsclient->s.origin[2] = z;
    switch (type) {
	case BOX: SP_item_box(fpsclient); break;
	case TOFU: SP_item_box(fpsclient); break;
	case BATTERY: SP_item_ammo(fpsclient); break;
	case GOLD: SP_item_grenades(fpsclient); break;
	case DATA: SP_item_datacd(fpsclient); break;
	case KRYPTONITE: SP_item_cube(fpsclient); break;
	case HEAD: SP_item_head(fpsclient); break;
    }
#endif
}

void set_quagent_type(int type) 
{
#ifdef TEST
    printf("quagent_type of %d is %d\n", config_num_quagents, type);
#else
    quagent_spawns[config_num_quagents].type = type;
#endif
}

void set_quagent_state(int state, float val)
{
#ifdef TEST
    printf("quagent_state(%d) of %d is %f\n", state, config_num_quagents, val);
#else
    switch (state) {
	case LIFETIME:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_LIFETIME;
	    quagent_spawns[config_num_quagents].lifetime = (int)val;
	    break;
	case INITIALWISDOM:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_WISDOM;
	    quagent_spawns[config_num_quagents].wisdom = (int)val;
	    break;
	case INITIALENERGY:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_ENERGY;
	    quagent_spawns[config_num_quagents].energy= val;
	    break;
	case INITIALWEALTH:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_WEALTH;
	    quagent_spawns[config_num_quagents].wealth= (int)val;
	    break;
	case INITIALHEALTH:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_HEALTH;
	    quagent_spawns[config_num_quagents].health= (int)val;
	    break;
	case ENERGYLOWTHRESHOLD:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_ENERGYLOW;
	    quagent_spawns[config_num_quagents].energy_low = val;
	    break;
	case AGEHIGHTHRESHOLD:
	    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_AGEHIGH;
	    quagent_spawns[config_num_quagents].age_high= (int)val;
	    break;
    }
#endif

}

void set_quagent_position(float x, float y, float z)
{
#ifdef TEST
    printf("quagent_position of %d is %f %f %f\n", config_num_quagents, x, y, z);
#else
    quagent_spawns[config_num_quagents].fields_set|=SPAWN_INFO_POSITION;
    quagent_spawns[config_num_quagents].position[0] = x;
    quagent_spawns[config_num_quagents].position[1] = y;
    quagent_spawns[config_num_quagents].position[2] = z;
#endif
}

#ifdef TEST
int main()
{
    yyparse();
}
#endif
