/*
 * Quagent
 *
 * $Log: quagent.c,v $
 * Revision 1.24  2004/11/09 19:52:43  costello
 * added runby to protocol and associated run_by functions to monster bot code
 *
 * Revision 1.23  2004/02/27 20:02:43  bh
 * Removed a sneaked in abort().
 *
 * Revision 1.22  2004/02/27 17:58:03  bh
 * Added TELL_ENERGYLOWNOT.
 *
 */


#include "quagent.h"
#include "quagent_items.h"

//socket includes
#include "sockhelp.h"
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <string.h>
#include <sys/wait.h>
#include <ctype.h>
#include <sys/time.h>
#include <fcntl.h>

edict_t *bot;

/* PROTOTYPES */

// calls quagent_do (for testing from console)
void callDo(edict_t *self);
// socket functions
void readSocket(edict_t *self);

void quagent_stand (edict_t *self)
{
    if(self->client->camera_on){
	// disable animation to eliminate the jittering
	//self->monsterinfo.currentmove = NULL;
	//return;
    }

    if(strstr(self->classname, "parasite")){
	self->monsterinfo.currentmove = &parasite_move_stand;
    } else if(strstr(self->classname, "gunner")){
	self->monsterinfo.currentmove = &gunner_move_stand;
    } else if(strstr(self->classname, "berserk")){
	self->monsterinfo.currentmove = &berserk_move_stand;
    } else if(strstr(self->classname, "chick")){
	self->monsterinfo.currentmove = &chick_move_stand;
    } else if(strstr(self->classname, "gladiator")){
	self->monsterinfo.currentmove = &gladiator_move_stand;
    } else if(strstr(self->classname, "infantry")){
	self->monsterinfo.currentmove = &infantry_move_stand;
    } else if(strstr(self->classname, "mutant")){
	self->monsterinfo.currentmove = &mutant_move_stand;
    } else if(strstr(self->classname, "float")){
	self->monsterinfo.currentmove = &floater_move_stand1;
    } else
	self->monsterinfo.currentmove = &soldier_move_stand1;
}

//
// DO actions ------------------------------------------------------------
//

/* callDo()
 * for testing quagent_do command parsing.  simulates a command string
 * comming in from somewhere (socket? console?) and being sent to quagent_do
 * for parsing and action.
 */
void callDo(edict_t *self){
    if(bot){
	char cmdstr[64];
	float yawvalue = bot->s.angles[YAW];//set to current direction

	Com_Printf("current yaw=%f\n",yawvalue);
	// let's generate a random direction for testing
	if (yawvalue > 10000.0){ 
	    //arbitrary upperbound
	    Com_Printf("reset yaw to 0.0\n");
	    yawvalue = 0.0;
	}
	
	if (rand() % 2 == 0){
	    yawvalue = (yawvalue+1)*25;
	}else{
	    yawvalue = (yawvalue+1)*(-25);
	}

	//randomly generate walk or stand action command
	int r;
	r = rand();
	
	Com_Printf("random=%d\n",r);
	if (rand() > 400000000){
	    sprintf(cmdstr,"DO run %f",yawvalue);
	}else{
	    sprintf(cmdstr,"DO stand %f",yawvalue);
	}
	Com_Printf("callDo calling quagent_do (%s)\n",cmdstr);
	quagent_do(bot,cmdstr);
    }else{
	Com_Printf("you need to spawn a quagent first.\n");
    }
}

/*
 * quagent_do
 *
 * According to the discussion led by GF on Jan 09 2004:
 * There are 2 categories of controller-issue requests
 * 1. DO <command>
 * 	<command> will be a string of the form:
 *	TurnBy <direction>
 * 	Stand
 * 	WalkBy <distance>
 * 	PickUp
 * 	Drop <object>
 * Besides the above, there are DO GET/SET <property> [value]
 * 2. ASK <something>
 * 	<something> will be of the form:
 * 	Radius <radius>
 * 	Rays <number of rays>
 * 
 * The robot will use
 * 	TELL <event>
 * to inform the controller an asynchronous event, such as the robot
 * hitting a wall or hearing sound, has occured
 *
 * For the requests, the response will be
 * 	OK/ERR <echo requests> <other data>
 *
 * quagent_do() should parse the command and cause the appropriate
 * activity and animation.
 *
 * All requests are case insensitive.
 *
 */
void quagent_do(edict_t *robot, char *cmd)
{
    char  *action;
    float val;
    int result = -1;
    char response[1024];
    static const char *delim = " \t\r\n";
    char *token;
    
    Com_Printf("quagent_do cmd=%s\n",cmd);

    // make a copy of cmd, which will be echoed back
    // the first 4 chars are saved for "OK  " or "ERR "
    response[0]='E';
    response[1]='R';
    response[2]='R';
    response[3]=' ';
    snprintf(response+4, 1019, "(%s)", cmd);

    // clear robot's message, so we'll know if robot has extra info
    // wants to sent back
    robot->quagentinfo.message[0] = '\0';

    //parse cmd string
    action = strtok(cmd, delim);
    
    if (action){
    if(strcasecmp(action, "do") == 0) {

	// commands
	action = strtok(NULL, delim);
	if(action){ // pointed out by Eric Hughs

	if (strcasecmp(action,"walkby") == 0 ){
	    token = strtok(NULL, delim);
	    if(token){
		float dist = atof(token);
		result = robot->quagentinfo.walk_by(robot, dist);
	    }else
		result = -1;

       }else if (strcasecmp(action,"runby") == 0 ){
         token = strtok(NULL, delim);
         if(token){
           float dist = atof(token);
           result = robot->quagentinfo.run_by(robot, dist);
         }else
           result = -1;

	}else if (strcasecmp(action,"turnby") == 0 ){
	    token = strtok(NULL, delim);

	    if(token) {
		float dyaw = atof(token);
		result = robot->quagentinfo.turn_by(robot, dyaw);
	    }else
		result = -1;

	}else if (strcasecmp(action,"setwalkspeed") == 0 ){
	    token = strtok(NULL, delim);

	    if(token) {
		float speed = atof(token);
		result = robot->quagentinfo.set_walk_speed(robot, speed);
	    }else
		result = -1;

	}else if ( strcasecmp(action,"stand") == 0 )  {

	    // void quagent_stand() 
	    robot->quagentinfo.stand(robot);
	    result = 1;

	}else if ( strcasecmp(action,"pickup") == 0 )  {
	    char *item = strtok(NULL, delim);
	    if(!item)
		result = -1;
	    else	
		result = robot->quagentinfo.pickup(robot,item);

	}else if ( strcasecmp(action,"drop") == 0 )  {

	    token = strtok(NULL, delim);
	    if(!token)
		result = -1;
	    else	
		result = robot->quagentinfo.putdown(robot, token);

	}else if (strcasecmp(action, "cameraon") == 0) {

	    result = robot->quagentinfo.camera_on(robot);

	}else if (strcasecmp(action, "cameraoff") == 0) {

	    result = robot->quagentinfo.camera_off(robot);

	}else if (strcasecmp(action, "look") == 0) {
	    char *image;
	    int size;
	    // 'image' command has binary data to send
	    result = robot->quagentinfo.image(robot, &size, &image);
	    if(result > 0) {
		response[0] = 'O';
		response[1] = 'K';
		response[2] = ' ';
		response[3] = ' ';
		// message is of the form "width height bytesperrow\n"
		// the trailing '\n' tells raw image data are coming
		strcat(response, robot->quagentinfo.message);
		sock_puts(robot->quagentinfo.connected_socket, response);
		sock_write(robot->quagentinfo.connected_socket, image, size);
		return;
	    }

	}else if (strcasecmp(action, "setalarm") == 0) {

	    token = strtok(NULL, delim);
	    if(token) {
		unsigned long mask = atol(token);
		result = robot->quagentinfo.set_alarm(robot, mask);
	    }else
		result = -1;

	}else if (strcasecmp(action, "getalarm") == 0) {

	    result = robot->quagentinfo.get_alarm(robot);

	}else if (strcasecmp(action, "getenergy") == 0) {

	    result = robot->quagentinfo.get_energy(robot);

	}else if (strcasecmp(action, "gethealth") == 0) {

	    result = robot->quagentinfo.get_health(robot);

	}else if (strcasecmp(action, "getwealth") == 0) {

	    result = robot->quagentinfo.get_wealth(robot);

	}else if (strcasecmp(action, "getwellbeing") == 0) {

	    result = robot->quagentinfo.get_wellbeing(robot);

	}else if (strcasecmp(action, "getwhere") == 0) {

	    result = robot->quagentinfo.get_where(robot);

	}else if (strcasecmp(action, "setagehighthreshold") == 0) {
	    token = strtok(NULL, delim);
	    if(token) {
		int age = atoi(token);
		result = robot->quagentinfo.set_age_threshold(robot, age);
	    }else
		result = -1;

	}else if (strcasecmp(action, "setenergylowthreshold") == 0) {
	    token = strtok(NULL, delim);
	    if(token) {
		float energy  = atof(token);
		result = robot->quagentinfo.set_energy_threshold(robot, energy);
	    }else
		result = -1;

	}else if (strcasecmp(action, "getinventory") == 0) {

	    result = robot->quagentinfo.get_inventory(robot);

	}else {
	    Com_Printf("Unknown command received (%s).\n",cmd);
	    result = -1;
	}
	}else
	    result = -1;

    } else if (strcasecmp(action, "ask") == 0) {

	// queries
	action = strtok(NULL, delim);
	if(action) {

	    if ( strcasecmp(action,"radius") == 0 )  {

		// XXX check if strtok returns NULL
		float rad; 
		unsigned long mask=1; //unused for now, set for a constant
		token = strtok(NULL, delim);
		if (token){
		    rad = atof(token);
		    result = robot->quagentinfo.tell_radius(robot, rad, mask);
		}else
		    result = -1;

	    }else if ( strcasecmp(action,"rays") == 0 )  {
		token = strtok(NULL, delim);
		if(token) {
		    int num_rays = atoi(token);
		    result = robot->quagentinfo.tell_ray(robot, num_rays);
		}else
		    result = -1;
	    }else {
		Com_Printf("Unknown query received (%s).\n",cmd);
		result = -1;
	    }
	}else
	    result = -1;
    }else {
	Com_Printf("Unknown request received (%s).\n",cmd);
	result = -1;
    }
    }

//end_dispatch:
    if(result > 0) {
	response[0] = 'O';
	response[1] = 'K';
	response[2] = ' ';
	response[3] = ' ';
    }

    // message must be a '\0' terminated c-string
    if(robot->quagentinfo.message[0])
	strcat(response, robot->quagentinfo.message);

    // append '\n' to the response XXX: check string length is in bound
    result = strlen(response);
    response[result] = '\n';
    response[result+1] = '\0';
    
    // send response to user
    sock_puts(robot->quagentinfo.connected_socket,response);

}

/**
 * pickup item itemname and add to inventory
 * returns 1 success 0 otherwise
 */
int pickup_item(edict_t *self, char *itemname){
    edict_t *ent = NULL;
    int radius = 60;
    char *quagent_name;

    while ((ent = findradius(ent, self->s.origin, radius)) != NULL) {
	if (ent == self){
	    continue;    
	}
	quagent_name = translate_quake2quagent(ent->classname);
	Com_Printf("Entity seen in %d radius: %s \n",radius,quagent_name);
	//if ( ent->item  && (strcmp(itemname,ent->item->pickup_name) == 0) ){
	if ( ent->item  && (strcmp(itemname,quagent_name) == 0) ){
	    //Com_Printf("quagent picking up item %s\n",ent->item->pickup_name);
	    Com_Printf("quagent picking up item %s\n",quagent_name);
	    /* set some flags to undraw the item on the screen
	     * @see g_items.c SetRespawn() for how game normally does this
	     * not commented or documented though...
	     */
	    ent->flags |= FL_RESPAWN;
	    ent->svflags |= SVF_NOCLIENT;
	    ent->solid = SOLID_NOT;
	    gi.linkentity (ent);

	    //add to inventory
	    item = ent->item;
	    self->quagentinfo.inventory[self->quagentinfo.invcount++] = item;
	    Com_Printf("inventory item %s saved invcount=%d\n",self->quagentinfo.inventory[self->quagentinfo.invcount-1]->pickup_name,self->quagentinfo.invcount);
	    // taking effects
	    if(strcasecmp(itemname, "tofu") == 0)
		self->health += 10;
	    else if(strcasecmp(itemname, "data") == 0){
		self->quagentinfo.wisdom+= 10;
		if(self->quagentinfo.wisdom > QUAGENT_MAX_WISDOM)
		    self->quagentinfo.wisdom = QUAGENT_MAX_WISDOM;
	    } else if(strcasecmp(itemname, "gold") == 0)
		self->quagentinfo.wealth += 10;
	    else if(strcasecmp(itemname, "battery") == 0)
		self->quagentinfo.energy += 10;
	    else if(strcasecmp(itemname, "kryptonite") == 0)
		self->quagentinfo.misc_states |= QSTATE_HAS_KRYPTO;
	    
	    return 1;
	}
    }
    //not picked up 
    return 0;
}   


//
// end Quagent DO functions --------------------------------------------



//
//Socket ---------------------------------------------------------------
//

/*
 * reads from the socket.  if there is data it will presumably be a 
 * command string coming from the outside instructing the bot to 'do'
 * something.  When the string is captured in the buffer it should be sent to 
 * a parse routing to call the appropriate 'do' command for the bot.
 * possible cmd string might look like "DO walk left" or "DO stand"
 * if it thinks the client has disconnected (a read error occurred)
 * it will close the socket.
 */
void readSocket(edict_t *self){
    char buffer[1024];
    char *current_character;
    int bytes = 0;

    if (self->quagentinfo.connected_socket <= 0){
	//no socket connection
	gi.dprintf("connected_socket < 0, shoudn't happen.\n");
	return;
    }

    bytes = sock_gets(self->quagentinfo.connected_socket, buffer, sizeof(buffer));
    if ( bytes == 0) {
	//nothing to read right now
    }else if(bytes < 0){
	//no connection
	// recognize if client is gone and disconnect socket @see sockhelp.c
	gi.dprintf("client gone.  closing socket. errno=%d\n",errno);
	//close(self->quagentinfo.connected_socket);
	//self->quagentinfo.connected_socket = 0;

	// no control connection so croak
	//quagent_die(self, NULL, NULL, 20, self->s.origin);
	// self->die will call quagentinfo.die
	// this mysterious code deserves some explanation:
	// self->die() should only be called once, it's in charge of
	// the dying animation, the real death is occured in *_dead(),
	// which is is the last call from die_movement.
	// if connected_socket is > 0, it means the quagent is still
	// alive till this time.  quagent_die, which will be called
	// in self->die(), will set connected_socket to -1.
	if(self->quagentinfo.connected_socket > 0) 
	    self->die(self, NULL, self, 10, self->s.origin);
	

    }else{
	//parse cmd and perform action
	//HB: sock_gets doesn't give us a '\0' c-string
	buffer[bytes] = '\0'; //bytes is less than 1024
	quagent_do(self,buffer);

	//add a newline on the buffer string
	//strcat(buffer, "\n");
	//fprintf(stderr,"buffer=%s\n",buffer);
	//echo back the cmd received on the socket
	//if (sock_puts(connected_socket, buffer) < 0) {
	    //connected = 0;
	//}
    }	    
}	    

void check_quagent_state(edict_t *self)
{
    char *msg;
    edict_t *ent=NULL;


    // if too old, die
    if(self->quagentinfo.age > self->quagentinfo.too_old)  {

	// put a dying message in the socket
	msg = "TELL DYING\n";
	sock_puts(self->quagentinfo.connected_socket, msg);
	close(self->quagentinfo.connected_socket);
	self->quagentinfo.connected_socket = -1;

	self->die(self, NULL, NULL, 20, self->s.origin);
    }

    if(self->quagentinfo.energy < self->quagentinfo.tell_weak_threshold) {
	if(!(self->quagentinfo.misc_states & QSTATE_ENERGY_LOW)){
	    self->quagentinfo.misc_states |= QSTATE_ENERGY_LOW;
	    msg = "TELL LOWENERGY\n";
	    sock_puts(self->quagentinfo.connected_socket, msg);
	}
    }else {
	if(self->quagentinfo.misc_states & QSTATE_ENERGY_LOW){
	    // energy has been lower than threshold
	    // get out of energy low status
	    self->quagentinfo.misc_states &= ~QSTATE_ENERGY_LOW;
	    msg = "TELL NOTLOWENERGY\n";
	    sock_puts(self->quagentinfo.connected_socket, msg);
	}
    }
    
    // growing older, what a pity
    self->quagentinfo.age ++; 

    // tell the user, it's time for some youth-fountain bath
    if(self->quagentinfo.age > self->quagentinfo.tell_old_threshold) {
	msg = "TELL OLDAGE\n";
	sock_puts(self->quagentinfo.connected_socket, msg);
	//XXX reset old_threshold, so this message is sent only once
	self->quagentinfo.tell_old_threshold = self->quagentinfo.too_old+1;
    }
    

    // tell radius for KRYPTONITE: every FREQ ticks to check or always 
    // check and every FREQ to tell: XXX
#define TELL_KRYPTONITE_FREQ 100
    while ((ent = findradius(ent, self->s.origin, 250)) != NULL) {
	char *quagent_name; //not the name of qagent, but name in quagent game
	vec3_t pos;
	float dist;
	if (ent == self){
	    continue;    
	}
	quagent_name = translate_quake2quagent(ent->classname);
	if (strcasecmp("kryptonite",quagent_name) == 0){

	    VectorSubtract(ent->s.origin, self->s.origin, pos);
	    dist = sqrt(pos[0]*pos[0]+pos[1]*pos[1]+pos[2]*pos[2]);
	    // kryptonite in near range
	    if(dist < 100 ) {
		// energy decreases extra .3
		self->quagentinfo.energy -= 0.3;
		if (!(self->quagentinfo.misc_states&QSTATE_KRYPTO_NEAR)) { 
		self->quagentinfo.misc_states |= QSTATE_KRYPTO_NEAR;
		msg = "TELL KRYPTONITENEAR\n";
		sock_puts(self->quagentinfo.connected_socket, msg);
		}
		//break;
		// can't break: what if there are multiple kryptonites?
		continue;
	    }
	    // kryptonite in far range
	    if(dist < 200 ) {
		// energy decreases extra .1
		self->quagentinfo.energy -= 0.1;

		if(self->quagentinfo.misc_states&QSTATE_KRYPTO_NEAR) {
		self->quagentinfo.misc_states &= ~QSTATE_KRYPTO_NEAR;
		self->quagentinfo.misc_states &= ~QSTATE_KRYPTO_FAR;
		}

		if(!(self->quagentinfo.misc_states&QSTATE_KRYPTO_FAR)) {
		self->quagentinfo.misc_states |= QSTATE_KRYPTO_FAR;
		msg = "TELL KRYPTONITEFAR\n";
		sock_puts(self->quagentinfo.connected_socket, msg);
		}
		continue;
	    }
	    // kryptonite out of range
	    if(self->quagentinfo.misc_states&QSTATE_KRYPTO_FAR &&
	      !(self->quagentinfo.misc_states&QSTATE_KRYPTO_NOT)) {
		self->quagentinfo.misc_states &= ~QSTATE_KRYPTO_FAR;
		self->quagentinfo.misc_states |= QSTATE_KRYPTO_NOT;
		msg = "TELL KRYPTONITEFARNOT\n";
		sock_puts(self->quagentinfo.connected_socket, msg);
	    }

	}
    }

    // if it has a kryptonite, the energy loses extra 0.3
    if(self->quagentinfo.misc_states & QSTATE_HAS_KRYPTO)
	self->quagentinfo.energy -= 0.3; 

    // energy decay: use a exponential function
    if(self->quagentinfo.energy >= 0)
	self->quagentinfo.energy -= 0.1; 
    else {
	msg = "TELL STALLING\n";
	sock_puts(self->quagentinfo.connected_socket, msg);
    }

}

/**
 * main quagent think function.
 * reads from checks for directive commands by reading the client
 * socket connection for the bot. updates energy and age info every click
 */ 
void mainthink(edict_t *self)
{


    // Read from connected socket look for command msgs
    if (self->quagentinfo.connected_socket > 0 ){
	//fprintf(stderr,"mainthink::read socket connection sock=%d\n",connected_socket);
	readSocket(self);
	check_quagent_state(self);
    }else{
	//fprintf(stderr,"%s has no socket connection to read from.\n",self->classname);
	return;
	//self->die(self, NULL, NULL, 20, self->s.origin);
    }

    // XXX: only if connected_socket > 0


    // note:if we explicitly set this in a tight think loop we circumvent
    // and freeze out animation thinks.
    //self->think = mainthink;
    //self->nextthink = level.time +2;
}  


/** 
*
* @fn edict_t *transfigure_quagent(edict_t *self)
* @brief Turn a monster to a quagent by filling the quagentinfo structure
* @param self The monster
*
* @return A whole new quagent
*
*/
edict_t *transfigure_quagent(edict_t *self)
{

    // don't kill the client
    self->monsterinfo.aiflags |= AI_GOOD_GUY;

    // along comes a quagent
    self->quagentinfo.connected_socket = -1; //not connected yet
    self->quagentinfo.energy = QUAGENT_ENERGY;
    self->quagentinfo.wealth = QUAGENT_WEALTH;
    self->quagentinfo.wisdom = QUAGENT_WISDOM;
    self->quagentinfo.age = 0;
    self->quagentinfo.too_old = QUAGENT_TOO_OLD;
    self->quagentinfo.tell_old_threshold = QUAGENT_TOO_OLD;
    self->quagentinfo.invcount = 0;  //come to this world clean
    self->quagentinfo.alarm_mask= ALR_ALL;  //very curious
    self->quagentinfo.misc_states = 0l;
    
    self->quagentinfo.trigger_alarm= quagent_trigger_alarm;
    self->quagentinfo.die = quagent_die;

    self->quagentinfo.turn_by = quagent_turn_by;
    self->quagentinfo.stand= quagent_stand;

    self->quagentinfo.pickup= pickup_item;
    self->quagentinfo.putdown= quagent_dropitem;
    self->quagentinfo.tell_radius= quagent_tell_radius;
    self->quagentinfo.tell_ray= quagent_tell_ray;
    self->quagentinfo.camera_on= quagent_camera_on;
    self->quagentinfo.camera_off= quagent_camera_off;
    self->quagentinfo.image= quagent_image;
    self->quagentinfo.set_alarm= quagent_set_alarm;
    self->quagentinfo.get_alarm= quagent_get_alarm;
    self->quagentinfo.set_walk_speed= quagent_set_walk_speed;
    self->quagentinfo.get_walk_speed= quagent_get_walk_speed;
    self->quagentinfo.get_health= quagent_get_health;
    self->quagentinfo.get_wealth= quagent_get_wealth;
    self->quagentinfo.get_energy= quagent_get_energy;
    self->quagentinfo.get_where= quagent_get_where;
    self->quagentinfo.get_age= quagent_get_age;
    self->quagentinfo.get_wellbeing= quagent_get_wellbeing;
    self->quagentinfo.get_inventory= quagent_get_inventory;
    self->quagentinfo.set_age_threshold= quagent_set_age_threshold;
    self->quagentinfo.set_energy_threshold= quagent_set_energy_threshold;

    self->quagentinfo.invcount = 0;
    
    return self;
}

