/*
 * #(@)quagent.h
 *
 * David Costello, costello@cs.rochester.edu, 10 Oct 2003
 * Time-stamp: <2004-01-16 15:15:21 costello>
 *
 * header file for quagent.c
 */

#include <stdarg.h>
#include "g_local.h"

// initial energy of a quaent
#define QUAGENT_ENERGY 			1000   
// initial wealth of a quaent
#define QUAGENT_WEALTH			0

// initial wisdom of a quaent
#define QUAGENT_WISDOM			0
// can't be more smarter than this
#define QUAGENT_MAX_WISDOM		100

// initial and ultimate maximum radius quagent can "ask radius"
#define QUAGENT_INIT_MAX_RADIUS		100
#define QUAGENT_ULT_MAX_RADIUS		400

// initial and ultimate maximum number of rays quagent can "ask rays"
#define QUAGENT_INIT_MAX_RAYS		4
#define QUAGENT_ULT_MAX_RAYS		72

// longest (in 10ms) a quagent can live
#define QUAGENT_TOO_OLD			10000

#define QSTATE_KRYPTO_NEAR              0x1
#define QSTATE_KRYPTO_FAR               0x2
#define QSTATE_KRYPTO_NOT               0x4
#define QSTATE_HAS_KRYPTO               0x8
#define QSTATE_ENERGY_LOW               0x10

#define ALR_MASK                        0x80
#define ALR_HELATHLOW                   0x81
#define ALR_ENERGYLOW                   0x82
#define ALR_OBJECTNEAR                  0x83
#define ALR_HITWALL                     0x84
#define ALR_HITOBJECTS                  0x85
#define ALR_SOUND                       0xF0
#define ALR_ALL                         0xFF

#define MIN(x,y) ((x)<(y)?(x):(y))

/************************************************
*
* Prototypes
*
************************************************/
//quagent_funcs.c
char *translate_quake2quagent(char *);
char *translate_quagent2quake(char *);
int quagent_trigger_alarm(edict_t*, unsigned char alarm, ...);
int quagent_die(edict_t*);
int quagent_turn_by(edict_t*, float yaw_angle);
int quagent_walk_by(edict_t*, float distance);
void quagent_stand(edict_t*);
int quagent_tell_radius(edict_t*, float radius, unsigned long mask);
int quagent_tell_ray (edict_t*, int num_rays);
int quagent_camera_on(edict_t*);
int quagent_camera_off(edict_t*);
int quagent_image (edict_t*, int *size, char **image);
int quagent_set_alarm(edict_t*, unsigned long alarm_mask);
int quagent_get_alarm(edict_t*);
int quagent_set_walk_speed(edict_t*, float speed);
int quagent_set_age_threshold(edict_t*, int threshold);
int quagent_set_energy_threshold(edict_t*, float threshold);
int quagent_get_walk_speed(edict_t*);
int quagent_get_health(edict_t*);
int quagent_get_wealth(edict_t*);
int quagent_get_energy(edict_t*);
int quagent_get_age(edict_t*);
int quagent_get_where(edict_t*);
int quagent_get_inventory(edict_t*);
int quagent_get_wellbeing(edict_t*);


/* quagent.c */

/* spawn */
edict_t* transfigure_quagent(edict_t *);
edict_t* SP_Quagent_ss(edict_t *self);
edict_t* SP_Quagent(edict_t *self);
edict_t* SP_Quagent_light(edict_t *self);
//void DAVEBOT_Respawn(edict_t *self);

/* AI functions */
void quagentai_stand (edict_t *self, float dist);
void quagentai_walk(edict_t *self, float dist);
void quagentai_run(edict_t *self, float dist);
qboolean quagentFindTarget (edict_t *self);

void quagent_do(edict_t * quagent, char *cmd); 
int pickup_item(edict_t *self, char *itemname);

/* think function */
void mainthink(edict_t *self);


//edict->touch
//void bot_touch(edict_t *self, edict_t *other, cplane_t *plane, csurface_t *surf);

/* animations */
extern mmove_t	parasite_move_stand;
extern mmove_t	gunner_move_stand;
extern mmove_t	soldier_move_stand1;
extern mmove_t berserk_move_stand;
extern mmove_t chick_move_stand;
extern mmove_t gladiator_move_stand;
extern mmove_t infantry_move_stand;
extern mmove_t medic_move_stand ;
extern mmove_t mutant_move_stand ;
extern mmove_t floater_move_stand1;	

/* spawn function for all sorts of quagents */
extern edict_t *SP_quagent_parasite(edict_t*);
extern edict_t *SP_quagent_gunner(edict_t*);
extern edict_t *SP_quagent_infantry(edict_t*);
extern edict_t *SP_quagent_chick(edict_t*);
extern edict_t *SP_quagent_gladiator(edict_t*);
extern edict_t *SP_quagent_berserk(edict_t*);
extern edict_t *SP_quagent_soldier_ss(edict_t*);
extern edict_t *SP_quagent_mutant(edict_t*);
extern edict_t* SP_quagent_floater (edict_t *);

/* information needed when spawning quagents from config files 
 * a subset of quagentinfo, which could be used directly, but ...
 */
typedef struct quagent_spawn_s {
    int type;
    unsigned int fields_set;
    int health;
    int wealth;
    int wisdom;
    int lifetime;
    float energy;
    float energy_low;
    int age_high;
    vec3_t position;
} quagent_spawn_info;

#define SPAWN_INFO_HEALTH		0x1
#define SPAWN_INFO_WEALTH		0x2
#define SPAWN_INFO_WISDOM		0x4
#define SPAWN_INFO_LIFETIME		0x8
#define SPAWN_INFO_ENERGY		0x10
#define SPAWN_INFO_ENERGYLOW		0x20
#define SPAWN_INFO_AGEHIGH		0x40
#define SPAWN_INFO_POSITION		0x80


// maximum number of quagents allowed in a config file
#define MAX_CONFIG_QUAGENTS		100

