/**************************************************
*
* $Log: quagent_config.c,v $
* Revision 1.1  2004/01/23 16:22:49  bh
* Parse configuration file.
*
*
**************************************************/

#include "quagent.h"
#include "quagent_config.h"

typedef struct item_spawn {
    char *item_name;
    void (*spawn_func)(edict_t *, float, float, float);
} item_spawn_t;

void spawn_box(edict_t*, float, float, float);
void spawn_tofu(edict_t*, float, float, float);
void spawn_battery(edict_t*, float, float, float);
void spawn_gold(edict_t*, float, float, float);
void spawn_data(edict_t*, float, float, float);
void spawn_kryptonite(edict_t*, float, float, float);

item_spawn_t item_spawns[] = {
    "BOX", spawn_box,
    "TOFU", spawn_tofu,
    "BATTERY", spawn_battery,
    "GOLD", spawn_gold,
    "DATA", spawn_data,
    "KRYPTONITE", spawn_kryptonite
};

int num_config_quagents = 0;
int config_quagent_types[MAX_CONFIG_QUAGENTS];

//#define NUM_ITEMS (sizeof(item_spawns)/sizeof(item_spawns[0]));
#define NUM_ITEMS  6

void spawn_box(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_box(ent);
}

void spawn_tofu(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_adrenaline(ent);
}

void spawn_battery(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_ammo(ent);
}

void spawn_gold(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_grenades(ent);
}

void spawn_data(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_datacd(ent);
}

void spawn_kryptonite(edict_t *ent, float x, float y, float z)
{
    // the origin of ent (client) was saved in init_quagent_configuration()
    ent->s.origin[0] = x;
    ent->s.origin[1] = y;
    ent->s.origin[2] = z;
    SP_item_cube(ent);
}


void spawn_items(edict_t *ent, char *item)
{
    static const char *delim=" \t";
    char *item_name, *x_value, *y_value, *z_value;
    int i;
    
    item_name = strtok(item, delim);
    x_value = strtok(NULL, delim);
    y_value = strtok(NULL, delim);
    z_value = strtok(NULL, delim);
    if(!item_name || !x_value || !y_value || !z_value) {
	fprintf(stderr, "Item config syntax error\n");
	return;
    }

    for(i=0; i<NUM_ITEMS; ++i) {

	if(strcasecmp(item_name, item_spawns[i].item_name) == 0) {
	    float x = atof(x_value);
	    float y = atof(y_value);
	    float z = atof(z_value);
	    (item_spawns[i].spawn_func)(ent, x, y, z);
	    break;
	}
    }

    if(i==NUM_ITEMS) 
	fprintf(stderr, "%s not a valid item\n", item_name);

}

void parse_item_config_file(edict_t *ent, FILE *fp)
{
    char line[1024];
    
    while(!feof(fp)) {

	//skip comments and empty lines
	do {
	    if(!fgets(line, 1023, fp))
		return;
	    
	}while(line[0] == '#' || line[0] == '\n');

	spawn_items(ent, line);
    }
}

void spawn_quagents(edict_t *ent, char *line)
{
    if (strncasecmp(line, "sold", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_SOLDIER;
    else if(strncasecmp(line, "para", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_PARASITE;
    else if(strncasecmp(line, "gunn", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_GUNNER;
    else if(strncasecmp(line, "chic", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_CHICK;
    else if(strncasecmp(line, "infa", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_INFANTRY;
    else if(strncasecmp(line, "glad", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_GLADIATOR;
    else if(strncasecmp(line, "bers", 4) == 0)
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_BERSERK;
    else
	config_quagent_types[num_config_quagents++] = QUAGENT_TYPE_SOLDIER;
}

void parse_quagent_config_file(edict_t *ent, FILE *fp)
{
    char line[1024];
    
    while(!feof(fp)) {

	//skip comments and empty lines
	do {
	    if(!fgets(line, 1023, fp))
		return;
	    
	}while(line[0] == '#' || line[0] == '\n');

	spawn_quagents(ent, line);
    }
}


