
/*
 * #(@)quagent_items.c
 *
 */

#include "quagent.h"
#include "quagent_items.h"


/**
 * defines a pickup function for an entity->item->pickup
 */
qboolean  db_pickup(struct edict_s *ent, struct edict_s *other){
    fprintf(stderr,"dbpickup\n");
    Com_Printf("Item %s picked-up by %s \n",ent->classname,other->classname);
    return true;
}

/**
 * defines a pickup function for an entity->item->pickup
 * the item instance(spawn) can assign this as it's pickup callback function
 */
qboolean  quagent_pickupitem(struct edict_s *ent, struct edict_s *other){

    Com_Printf("Item %s picked-up by %s \n",ent->classname,other->classname);

    item = ent->item;
    other->quagentinfo.inventory[other->quagentinfo.invcount++] = item;

    Com_Printf("inventory item %s saved invcount=%d\n",other->quagentinfo.inventory[other->quagentinfo.invcount-1]->pickup_name,other->quagentinfo.invcount);

    return true;
}

/**
 * drop an item with 'itemname' from our inventory on the floor 
 * returns 1 success 0 otherwise
 */
int quagent_dropitem(edict_t *self, char *itemname){
    int		i;
    gitem_t	*it;
    char *quagent_name;
    
    for (i=0 ; i<self->quagentinfo.invcount ; i++){
	it = self->quagentinfo.inventory[i];
	if (!it->pickup_name)
	    continue;
	quagent_name = translate_quake2quagent(it->pickup_name);
	if(quagent_name == it->pickup_name) {
	    // shouldn't happen, item->pickup_name should be the same
	    // XXX OWNER owner->classname and translate_quake2quagent()
	    // works by classname, not pickup_name;
	    gi.dprintf("****************** ATTN: pickup_name anomaly: %s\n",it->pickup_name);
	}
	if (Q_stricmp(quagent_name, itemname) == 0){
	    Com_Printf("dropping item %s found in inventory at index=%d\n",it->pickup_name,i);

	    // taking effects
	    if(strcasecmp(itemname, "tofu") == 0)
		self->health -= 10;
	    else if(strcasecmp(itemname, "data") == 0)
		self->quagentinfo.wisdom-= 10;
	    else if(strcasecmp(itemname, "gold") == 0)
		self->quagentinfo.wealth -= 10;
	    else if(strcasecmp(itemname, "battery") == 0)
		self->quagentinfo.energy -= 10;
	    else if(strcasecmp(itemname, "kryptonite") == 0)
		self->quagentinfo.misc_states &= ~QSTATE_HAS_KRYPTO;
	    

	    // DC doesn't work for Health box (shutdown error?)
	    // HB fixed: see changelog
	    Drop_Item(self,it);
	    remove_inventory_item(self,i);
	    return 1;
	}
    }
    Com_Printf("Drop item %s NOT FOUND in inventory\n",itemname);	
    return 0;
}


void drop_last_item(edict_t *self){

    if(lastitem){
	Drop_Item(self,lastitem);
    }else{
	Com_Printf("No item to drop\n");
    }
}

/**
 * removes an item from the inventory list
 * when it has been dropped
 */
void remove_inventory_item(edict_t *self, int index){
    int i;
    Com_Printf("removing inventory item at index:%d.\n",index);
    //shift remaining inventory towards front of the list
    if (self->quagentinfo.invcount > 1){
	for(i=index;i<self->quagentinfo.invcount; i++){
	    self->quagentinfo.inventory[i] = self->quagentinfo.inventory[i+1];
	}
    }else{
	self->quagentinfo.inventory[0]=NULL;
    }
    self->quagentinfo.invcount -= 1;
    
}

//
// Item Spawns
//

/**
 * spawns a large health box
 */
void SP_item_box (edict_t *self){

    gi.dprintf("SP_item_box\n");

    edict_t *health;
    gitem_t *healthbox;
    vec3_t  offset;

    healthbox = FindItem("Health");

    health = G_Spawn();
    health->classname = "Health";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, health->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, health->s.angles);

    health->model = "models/items/healing/large/tris.md2";
    health->count = 25; //health value

    VectorSet (health->mins, -16, -16, -24);
    VectorSet (health->maxs, 16, 16, 24);

    SpawnItem (health, FindItem ("Health"));

    //health->s.origin[1]+=90;//move drop point forward
    //health->s.origin[0]-=100;//move drop point left

    //set pickup function callback
    health->item->pickup = quagent_pickupitem; 


    gi.soundindex ("items/l_health.wav");
    gi.linkentity(health);

    lastitem = health->item;

}

/**
 * spawns an ammo backpack
 */
void SP_item_ammo (edict_t *self){

    gi.dprintf("SP_item_ammo\n");

    edict_t *ammo;
    gitem_t *ammopack;
    vec3_t  offset;

    ammopack = FindItem("Ammo Pack");

    ammo = G_Spawn();
    ammo->classname = "Ammo Pack";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, ammo->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, ammo->s.angles);

    ammo->model = "models/items/pack/tris.md2";
    ammo->count = 25; //ammo value

    VectorSet (ammo->mins, -16, -16, -24);
    VectorSet (ammo->maxs, 16, 16, 24);

    SpawnItem (ammo, FindItem ("Ammo Pack"));

    //ammo->s.origin[1]+=90;//move drop point forward
    //ammo->s.origin[0]-=100;//move drop point left

    //set pickup function callback
    ammo->item->pickup = quagent_pickupitem; 
    /* create a pickup_name (no spaces for easier parsing)
     * if we don't explictly set this we get "Power Cube"
     */
    //ammo->item->pickup_name = "ammopack";

    gi.soundindex ("items/pkup.wav");
    gi.linkentity(ammo);

    lastitem = ammo->item;

}

/**
 * spawns adrenaline
 */
void SP_item_adrenaline (edict_t *self){

    gi.dprintf("SP_item_adrenaline\n");

    edict_t *adrenaline;
    gitem_t *adrenalinebox;
    vec3_t  offset;

    adrenalinebox = FindItem("Adrenaline");

    adrenaline = G_Spawn();
    adrenaline->classname = "Adrenaline";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, adrenaline->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, adrenaline->s.angles);

    adrenaline->model = "models/items/adrenal/tris.md2";
    adrenaline->count = 25; //adrenaline value

    VectorSet (adrenaline->mins, -16, -16, -24);
    VectorSet (adrenaline->maxs, 16, 16, 24);

    SpawnItem (adrenaline, FindItem ("Adrenaline"));

    //adrenaline->s.origin[1]+=90;//move drop point forward
    //adrenaline->s.origin[0]-=100;//move drop point left


    //set pickup callback function
    adrenaline->item->pickup = quagent_pickupitem; 


    gi.soundindex ("items/pkup.wav");
    gi.linkentity(adrenaline);
    lastitem = adrenaline->item;

}

/**
 * spawns a power cube
 */
void SP_item_cube (edict_t *self){

    gi.dprintf("SP_item_cube\n");

    edict_t *cube;
    gitem_t *cubebox;
    vec3_t  offset;

    cubebox = FindItem("Power Cube");

    cube = G_Spawn();
    cube->classname = "Power Cube";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, cube->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, cube->s.angles);

    cube->model = "models/items/keys/power/tris.md2";
    cube->count = 25; //cube value

    VectorSet (cube->mins, -16, -16, -24);
    VectorSet (cube->maxs, 16, 16, 24);

    SpawnItem (cube, FindItem ("Power Cube"));

    //cube->s.origin[1]+=90;//move drop point forward
    //cube->s.origin[0]-=100;//move drop point left


    //set pickup function callback
    cube->item->pickup = quagent_pickupitem; 
    /* create a pickup_name (no spaces for easier parsing)
     * if we don't explictly set this we get "Power Cube"
     */
   //cube->item->pickup_name = "powercube";

    gi.soundindex ("items/pkup.wav");
    gi.linkentity(cube);

    lastitem = cube->item;

}

/**
 * spawns a CD
 */
void SP_item_datacd (edict_t *self){

    gi.dprintf("SP_item_datacd\n");

    edict_t *datacd;
    gitem_t *datacdbox;
    vec3_t  offset;

    datacdbox = FindItem("Data CD");

    datacd = G_Spawn();
    datacd->classname = "Data CD";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, datacd->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, datacd->s.angles);

    datacd->model = "models/items/keys/data_cd/tris.md2";
    datacd->count = 25; //datacd value

    VectorSet (datacd->mins, -16, -16, -24);
    VectorSet (datacd->maxs, 16, 16, 24);

    SpawnItem (datacd, FindItem ("Data CD"));

    //datacd->s.origin[1]+=90;//move drop point forward
    //datacd->s.origin[0]-=100;//move drop point left

    //set pickup function callback
    datacd->item->pickup = quagent_pickupitem; 
    /* create a pickup_name (no spaces for easier parsing)
     * if we don't explictly set this we get "Power Cube"
     *
     * HB: we can't set pickup_name: it'll make
     *   FindItem("Data CD") fail.
     */
    //datacd->item->pickup_name = "datacd";

    gi.soundindex ("items/pkup.wav");
    gi.linkentity(datacd);

    lastitem = datacd->item;

}

/**
 * spawns grenades
 */
void SP_item_grenades (edict_t *self){

    gi.dprintf("SP_item_grenades\n");

    edict_t *grenades;
    gitem_t *grenadesbox;
    vec3_t  offset;

    grenadesbox = FindItem("Grenades");

    grenades = G_Spawn();
    grenades->classname = "Grenades";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, grenades->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, grenades->s.angles);

    grenades->model = "models/items/ammo/grenades/medium/tris.md2";
    grenades->count = 25; //grenades value

    VectorSet (grenades->mins, -16, -16, -24);
    VectorSet (grenades->maxs, 16, 16, 24);

    SpawnItem (grenades, FindItem ("Grenades"));

    //grenades->s.origin[1]+=90;//move drop point forward
    //grenades->s.origin[0]-=100;//move drop point left

    //set pickup function callback
    grenades->item->pickup = quagent_pickupitem; 
    /* create a pickup_name (no spaces for easier parsing)
     * if we don't explictly set this we get "Power Cube"
     */
    grenades->item->pickup_name = "grenades";

    gi.soundindex ("items/pkup.wav");
    gi.linkentity(grenades);

    lastitem = grenades->item;

}

/**
 * spawns ancient head
 */
void SP_item_head (edict_t *self){

    gi.dprintf("SP_item_head\n");

    edict_t *head;
    gitem_t *headbox;
    vec3_t  offset;

    headbox = FindItem("Ancient Head");

    head = G_Spawn();
    head->classname = "Ancient Head";

    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, head->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, head->s.angles);

    head->model = "models/items/c_head/tris.md2";
    head->count = 25; //head value

    VectorSet (head->mins, -16, -16, -24);
    VectorSet (head->maxs, 16, 16, 24);

    SpawnItem (head, FindItem ("Ancient Head"));

    //head->s.origin[1]+=90;//move drop point forward
    //head->s.origin[0]-=100;//move drop point left

    //set pickup function callback
    head->item->pickup = quagent_pickupitem; 
    /* create a pickup_name (no spaces for easier parsing)
     * if we don't explictly set this we get "Power Cube"
     */
    //head->item->pickup_name = "head";

    gi.soundindex ("items/pkup.wav");
    gi.linkentity(head);

    lastitem = head->item;

}

/**
 * allows barrel to be pushed
 */
void barreltouch (edict_t *self, edict_t *other, cplane_t *plane, csurface_t *surf){
	float	ratio;
	vec3_t	v;

	if ((!other->groundentity) || (other->groundentity == self))
		return;

	ratio = (float)other->mass / (float)self->mass;
	VectorSubtract (self->s.origin, other->s.origin, v);
	M_walkmove (self, vectoyaw(v), 20 * ratio * FRAMETIME);
}

/**
 * for barrel->die callback
 */
void barreldelay (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point){
	self->takedamage = DAMAGE_NO;
	//self->nextthink = level.time + 2 * FRAMETIME;
	//self->think = barrel_explode;
	//self->activator = attacker;
}
void SP_explobox (edict_t *self){

    gi.dprintf("SP_explobox\n");

    edict_t *barrel;
    gitem_t *barrelbox;
    vec3_t  offset;

    barrel = G_Spawn();
    barrel->classname = "misc_explobox";
    //drop the box infront of "self"
    //VectorSet(offset, 40, 40, 0);
    VectorAdd(self->s.origin, offset, offset);
    VectorCopy(offset, barrel->s.origin);
    VectorCopy(self->s.origin, offset);
    VectorCopy(self->s.angles, barrel->s.angles);

    gi.modelindex ("models/objects/debris1/tris.md2");
    gi.modelindex ("models/objects/debris2/tris.md2");
    gi.modelindex ("models/objects/debris3/tris.md2");
    
    barrel->solid = SOLID_BBOX;
    barrel->movetype = MOVETYPE_STEP;

    barrel->model = "models/objects/barrels/tris.md2";
    barrel->s.modelindex = gi.modelindex (barrel->model);
    VectorSet (barrel->mins, -16, -16, 0);
    VectorSet (barrel->maxs, 16, 16, 40);
    
    if (!barrel->mass)
	barrel->mass = 50;
    if (!barrel->health)
	barrel->health = 10;
    if (!barrel->dmg)
	barrel->dmg = 150;
    
    barrel->die = barreldelay;
    barrel->takedamage = DAMAGE_YES;
    barrel->monsterinfo.aiflags = AI_NOSTEP;
    
    barrel->touch = barreltouch;
    barrel->think = M_droptofloor;
    barrel->nextthink = level.time + 2 * FRAMETIME;
    
    gi.linkentity (barrel);
}
