/*
 * #(@) quagent_server.c
 *
 * David Costello, costello@cs.rochester.edu, 16 Jan 2004
 * Time-stamp: <2004-01-20 12:45:06 costello>
 *
 * listens on advertised port for quagent client connections
 * and spawns a new quagent into the game for each accepted connection.
 */

#include "quagent.h"
#include "quagent_server.h"
#include "y.tab.h"
#include <math.h>
#define D2R(a) ((a)*M_PI/180.)

int listening_socket = -1;
char *listen_port = "33333";
int clientsocket = 0;

edict_t *quagent_server; //just one instance of quagent_server 
edict_t *fpsclient; //first person game client

// XXX: relies on the fact/assumption the struct will be set 0 by 
// the compiler.  Or else, we have to do it by ourselves.
int config_num_quagents = 0;
quagent_spawn_info quagent_spawns[MAX_CONFIG_QUAGENTS];

// default quagent states
quagent_spawn_info parasite_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info berserk_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info chick_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info gladiator_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info gunner_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info infantry_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info soldier_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info mutant_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};
quagent_spawn_info float_info={0,0,100,0,0,5000,1000.f,5.f,4000, {0,0,0}};

//
// SPAWN
//

void SP_Quagent_Server (edict_t *initclient){

    //edict_t *self;
    //fpsclient = initclient;
    vec3_t   spawn_origin, spawn_angles;

    // spawn an entity for quagent_server
    quagent_server = G_Spawn();
    //SelectSpawnPoint (self, spawn_origin, spawn_angles);

    quagent_server->classname = "quagent_server";

    //VectorSet (self->mins, -16, -16, -24);
    //VectorSet (self->maxs, 16, 16, 32);
    quagent_server->movetype = MOVETYPE_NONE;
    quagent_server->solid = SOLID_NOT;

    gi.dprintf("quagent_server: location origin %s\n", vtos(quagent_server->s.origin));
    
    // register this new entity with the game engine (world)
    gi.linkentity (quagent_server);

    // kickoff think loop waiting for connections
    qserver_think(quagent_server);
}

/**
 * set a socket to non blocking mode
 */
void setnonblocking(int sock){
	int opts;

	opts = fcntl(sock,F_GETFL);
	if (opts < 0) {
		perror("fcntl(F_GETFL)");
		exit(EXIT_FAILURE);
	}
	opts = (opts | O_NONBLOCK);
	if (fcntl(sock,F_SETFL,opts) < 0) {
		perror("fcntl(F_SETFL)");
		exit(EXIT_FAILURE);
	}else{
	    fprintf(stderr,"nonblocking set\n");
	}
	return;
}

/** 
*
* @fn void load_default_info()
* @brief Load default quagentinfo values for each type of quagents.
* @param 
*
*/
void load_default_info(edict_t *bot, quagent_spawn_info *spawn_info)
{
    quagentinfo_t *info = &(bot->quagentinfo);

    bot->health = spawn_info->health;
    info->wealth = spawn_info->wealth;
    info->wisdom = spawn_info->wisdom;
    info->energy = spawn_info->energy;
    info->tell_weak_threshold = spawn_info->energy_low;
    info->too_old = spawn_info->lifetime;
    info->tell_old_threshold = spawn_info->age_high;
}

/** 
*
* @fn void load_quagentinfo()
* @brief Load quagentinfo values from the configfile, with some sanity check.
* @param 
*
*
*/
void load_quagentinfo(edict_t *bot, quagent_spawn_info *spawn_info)
{
    quagentinfo_t *info = &(bot->quagentinfo);

    if(spawn_info->fields_set & SPAWN_INFO_HEALTH)
	bot->health = spawn_info->health;

    if(spawn_info->fields_set & SPAWN_INFO_WEALTH)
	info->wealth = spawn_info->wealth;

    if(spawn_info->fields_set & SPAWN_INFO_ENERGY)
	info->energy = spawn_info->energy;

    if(spawn_info->fields_set & SPAWN_INFO_ENERGYLOW){
	if( spawn_info->energy_low < spawn_info->energy && 
	    spawn_info->energy_low>0)
	    info->tell_weak_threshold = spawn_info->energy_low;
	else
	    info->tell_weak_threshold = 0;
    }

    if(spawn_info->fields_set & SPAWN_INFO_LIFETIME)
	info->too_old = spawn_info->lifetime;

    if(spawn_info->fields_set & SPAWN_INFO_AGEHIGH){
	if( spawn_info->age_high > 0 && 
	    spawn_info->age_high < spawn_info->lifetime)
	    info->tell_old_threshold = spawn_info->age_high;
	else
	    info->tell_old_threshold = info->too_old;
    }

    if(spawn_info->fields_set & SPAWN_INFO_WISDOM){
	if(spawn_info->wisdom < QUAGENT_MAX_WISDOM)
	    info->wisdom = spawn_info->wisdom;
	else
	    info->wisdom = QUAGENT_MAX_WISDOM;
    }
}

/* dc
 * for each new connection spawn a new quagent and hand it the 
 * new socket filedescriptor so the initiating connection can continue to 
 * communicate with it.  If the connection is broken the is no way to 
 * reconnect with that bot. We will address this later.
 */
void qserver_think(edict_t *self){

    struct sockaddr_in address;
    int reuse_addr = 1;
    int socket_type = SOCK_STREAM;
    int port;
    edict_t *newbot;
    static int current_quagent=0;

    //fprintf(stderr,"qserver_think()...\n");

    //
    // init and bind listening socket
    //
    if (listening_socket < 0){
	
	port = atoport(listen_port,"tcp");

	fprintf(stderr,"qserver_think::initsocket port=%d\n",port);

        /* Setup internet address information.  
         This is used with the bind() call */
	memset((char *) &address, 0, sizeof(address));
	address.sin_family = AF_INET;
	address.sin_port = port;
	address.sin_addr.s_addr = htonl(INADDR_ANY);
      
	listening_socket = socket(AF_INET, socket_type, 0);
	if (listening_socket < 0) {
	    perror("socket");
	    exit(EXIT_FAILURE);
	}
	
	setsockopt(listening_socket, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, 
		   sizeof(reuse_addr));
	    

        setnonblocking(listening_socket);	      
      
	if (bind(listening_socket, (struct sockaddr *) &address, 
	       sizeof(address)) < 0) {
	    perror("bind");
	    close(listening_socket);
	    exit(EXIT_FAILURE);
	}
	fprintf(stderr,"listening_socket = %d\n",listening_socket);
      
	if (socket_type == SOCK_STREAM) {
	    listen(listening_socket, 1);
	    fprintf(stderr,"listening...\n");
	}
    }

    //
    // accept new connections
    //
    if (clientsocket <= 0){

	//fprintf(stderr,"qserver_think::accept connection\n");
	clientsocket = accept(listening_socket, NULL, NULL);
	//fprintf(stderr,"qserver_think::after accept call\n");

	if (clientsocket < 0) {
	    /* Either a real error occured, or blocking was interrupted for
	       some reason.  Only abort execution if a real error occured. */
	    if (errno != EINTR && errno != EAGAIN) {
		perror("accept");
		close(listening_socket);
		exit(EXIT_FAILURE);
	    }
	}else{
	    vec3_t client_orig;

	    //we've connected so spawn a bot and hand it the new connection
	    setnonblocking(clientsocket);
	    fprintf(stderr,"clientsocket set to nonblocking\n");

	    // save client's origin
	    VectorCopy(fpsclient->s.origin, client_orig);

#define SPAWN_DISTANCE 60

	    //spawn a quagent 
	    if(current_quagent < config_num_quagents) {
		// if init position is given, copy it to fpsclient
		if(quagent_spawns[current_quagent].fields_set &
			SPAWN_INFO_POSITION)
		    VectorCopy(quagent_spawns[current_quagent].position,
			    fpsclient->s.origin);
		else{
		    float yaw = fpsclient->s.angles[YAW];
		    fpsclient->s.origin[0] += SPAWN_DISTANCE*cos(D2R(yaw));
		    fpsclient->s.origin[1] += SPAWN_DISTANCE*sin(D2R(yaw));
		}

		switch(quagent_spawns[current_quagent].type){
		    case PARASITE:
			newbot = SP_quagent_parasite(fpsclient);
			load_default_info(newbot, &parasite_info);
			break;
		    case GUNNER:
			newbot = SP_quagent_gunner(fpsclient);
			load_default_info(newbot, &gunner_info);
			break;
		    case CHICK:
			newbot = SP_quagent_chick(fpsclient);
			load_default_info(newbot, &chick_info);
			break;
		    case GLADIATOR:
			newbot = SP_quagent_gladiator(fpsclient);
			load_default_info(newbot, &gladiator_info);
			break;
		    case BERSERK:
			newbot = SP_quagent_berserk(fpsclient);
			load_default_info(newbot, &berserk_info);
			break;
		    case INFANTRY:
			newbot = SP_quagent_infantry(fpsclient);
			load_default_info(newbot, &infantry_info);
			break;
		    case MUTANT:
			newbot = SP_quagent_mutant(fpsclient);
			load_default_info(newbot, &mutant_info);
			break;
		    case FLOAT:
			newbot = SP_quagent_floater(fpsclient);
			load_default_info(newbot, &float_info);
			break;
		    default:
			//newbot = SP_Quagent_ss(fpsclient);
			newbot = SP_quagent_soldier_ss(fpsclient);
			load_default_info(newbot, &soldier_info);
			break;
		}


		load_quagentinfo(newbot, &(quagent_spawns[current_quagent]));
		/*
		newbot->health = quagent_spawns[current_quagent].health;
		if(newbot->health <= 0)
		    newbot->health = 10;
		 */
		current_quagent ++;

	    }else {
		float yaw = fpsclient->s.angles[YAW];
		fpsclient->s.origin[0] += SPAWN_DISTANCE*cos(D2R(yaw));
		fpsclient->s.origin[1] += SPAWN_DISTANCE*sin(D2R(yaw));
		newbot = SP_quagent_soldier_ss(fpsclient);
		load_default_info(newbot, &soldier_info);

	    }
	    newbot->quagentinfo.connected_socket = clientsocket;
	    sock_puts(newbot->quagentinfo.connected_socket,"Connected to quakeII bot.\n");
	    
	    // restore client's origin
	    VectorCopy(client_orig, fpsclient->s.origin);

	    //okay ready for new connection
	    clientsocket = 0;
	}
    }

    self->think = qserver_think;
    self->nextthink = level.time +2;
}
	
/**
 * called at game startup time from p_client::ClientBegin
 *
 *   Spawns items for this session and starts the quagent_server
 *   for spawning bots
 */
extern FILE *yyin;
extern int yyparse();
void init_quagent_configuration(edict_t *ent){

    fpsclient = ent;
    if(yyin=fopen("quagent.config", "r")) {
	vec3_t client_orig;
	// save client's origin for item spawns
	VectorCopy(fpsclient->s.origin, client_orig);

	yyparse();
	
	// restore client's origin
	VectorCopy(client_orig, fpsclient->s.origin);

	fclose(yyin);
    }

    // start quagent_server listening for client connections
    SP_Quagent_Server(ent);

}

