% COMANDI DISPONIBILI: 	TURN, WALK, STAND, RADIUS, RAYS, PICK
% STATI DISPONIBILI: 		SOPPIATTO, FUGA


% nextCmd(+PrevCmdStory,+TellState,+CmdResponse,+PrevAgentState,
%	-NextCmd,-NextPar,-NextCmdStory,-NextAgentState)
%		PrevCmdStory: storico attuale dei comandi eseguiti
%		TellState: stato attuale della risposta TELL
%		CmdResponse: lista contenente la risposta all'ultimo comando eseguito
%		PrevAgentState: stato attuale dell'agente
% 		NextCmd: prossimo comando da eseguire
%		NextPar: parametro del comando da eseguire (0 se non applicabile)
%		NextCmdStory: storico aggiornato dei comandi eseguiti
%		NextAgentState: nuovo stato dell'agente dopo l'esecuzione del comando


%nextCmd([start],_,_,STATE,CMD,PAR,STOR,STATE):- CMD is turn, PAR is 0, STOR is [cmd(turn, 0)].
nextCmd([start],_,_,STATE,CMD,PAR,STOR,STATE):- CMD is turn, PAR is 180, STOR is [cmd(turn, 180)].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MODALITA' SOPPIATTO

% Dopo una rotazione cammina in avanti
nextCmd([cmd(turn,ARG)|PSTOR],_,_,soppiatto,CMD,PAR,STOR,STATE):- CMD is walk, PAR is 20, STATE is soppiatto, 
	union([cmd(turn,ARG)],PSTOR,STOR1), union([cmd(walk,20)],STOR1,STOR).
% Dopo aver camminato controlla la presenza della guardia e dell'obiettivo
nextCmd([cmd(walk,ARG)|PSTOR],_,_,soppiatto,CMD,PAR,STOR,STATE):- CMD is radius, PAR is 200, STATE is soppiatto, 
	union([cmd(walk,ARG)],PSTOR,STOR1), union([cmd(radius,200)],STOR1,STOR).	
% Dopo un ciclo di attesa controlla la presenza della guardia e dell'obiettivo
nextCmd([cmd(stand,ARG)|PSTOR],_,_,soppiatto,CMD,PAR,STOR,STATE):- CMD is radius, PAR is 200, STATE is soppiatto, 
	union([cmd(stand,ARG)],PSTOR,STOR1), union([cmd(radius,200)],STOR1,STOR).	
% Nessun oggetto in vista: controlla la distanza dagli ostacoli
nextCmd([cmd(radius,ARG)|PSTOR],_,[],soppiatto,CMD,PAR,STOR,STATE):- CMD is rays, PAR is 4, STATE is soppiatto,
	union([cmd(radius,ARG)],PSTOR,STOR1), union([cmd(rays,4)],STOR1,STOR).
	
% Guardia in vista: attende
nextCmd([cmd(radius,ARG)|PSTOR],_,RESP,soppiatto,CMD,PAR,STOR,STATE):-
	findall(K,member(obj(michael,X,_,_),RESP,_),L), length(L,N), N==1, CMD is stand, PAR is 0, STATE is soppiatto, 
	union([cmd(radius,ARG)], PSTOR,STOR1), union([cmd(stand,0)],STOR1,STOR).
	
% Obiettivo in vista: inizia l'avvicinamento
nextCmd([cmd(radius,ARG)|PSTOR],_,RESP,soppiatto,CMD,PAR,STOR,STATE):-
	findall(K,member(obj(data,X,_,_),RESP,_),L), length(L,N), N==1, getDistance(RESP,X1,Y1), 
	D1 is sqrt((X1*X1)+(Y1*Y1)), D1@>50, A is (atan(Y1/X1)*180)/3.1415926535, corrAng(A,X1,Y1,A1),
	CMD is turn, PAR is A1, STATE is soppiatto, 
	union([cmd(radius,ARG)], PSTOR,STOR1), union([cmd(turn,A1)],STOR1,STOR).
% Obiettivo raggiunto: lo preleva e passa in modalità fuga
nextCmd([cmd(radius,ARG)|PSTOR],_,RESP,soppiatto,CMD,PAR,STOR,STATE):-
	findall(K,member(obj(data,X,_,_),RESP,_),L), length(L,N), N==1, getDistance(RESP,X1,Y1), 
	D1 is sqrt((X1*X1)+(Y1*Y1)), D1@=<50, CMD is pick, PAR is data, STATE is fuga, 
	union([cmd(radius,ARG)], PSTOR,STOR1), union([cmd(pick,data)],STOR1,STOR).
	
% Oggetti non rilevanti in vista: controlla la distanza dagli ostacoli
nextCmd([cmd(radius,ARG)|PSTOR],_,_,soppiatto,CMD,PAR,STOR,STATE):- CMD is rays, PAR is 4, STATE is soppiatto,
	union([cmd(radius,ARG)],PSTOR,STOR1), union([cmd(rays,4)],STOR1,STOR).
	
% Lettura dei 4 raggi: sceglie la direzione da seguire ed avanza
nextCmd([cmd(rays,4)|PSTOR],_,[obj(_,X1,Y1,_),obj(_,X2,Y2,_),obj(_,X3,Y3,_),obj(_,X4,Y4,_)],soppiatto,CMD,PAR,STOR,STATE):- 
	selectDirection(X1,Y1,X2,Y2,X4,Y4,CMD,PAR), STATE is soppiatto,
	union([cmd(rays,4)],PSTOR,STOR1), union([cmd(CMD,PAR)],STOR1,STOR).
% Lettura di un numero di raggi superiore a 4: azione non valida
nextCmd([cmd(rays,ARG)|PSTOR],_,_,soppiatto,CMD,PAR,STOR,STATE):- CMD is stand, PAR is 0, STATE is soppiatto,
	union([cmd(rays,ARG)],PSTOR,STOR1), union([cmd(stand,0)],STOR1,STOR).

% Calcola la distanza degli ostacoli nelle tre direzioni rilevanti (davanti, a sinistra, a destra) e seleziona la direzione
% in cui gli ostacoli sono più lontani
selectDirection(X1,Y1,X2,Y2,X3,Y3,CMD,PAR):-
	D1 is sqrt((X1*X1)+(Y1*Y1)), D2 is sqrt((X2*X2)+(Y2*Y2)), D3 is sqrt((X3*X3)+(Y3*Y3)), whatDir(D1,D2,D3,CMD,PAR).
whatDir(D1,D2,D3,CMD,PAR):- D1@>D2, D1@>D3, CMD is walk, PAR is 20.
whatDir(D1,D2,D3,CMD,PAR):- D2@>D1, D2@>D3, CMD is turn, PAR is 90.
whatDir(D1,D2,D3,CMD,PAR):- D3@>D1, D3@>D2, CMD is turn, PAR is -90.
whatDir(D1,D2,D3,CMD,PAR):- CMD is walk, PAR is 20.

getDistance(L,X1,Y1):- member(obj(data,X1,Y1,_),L,_).

% Corregge l'angolo determinato in base al quadrante in cui ci si trova
corrAng(A,X,Y,A1):- Y==0, X@>0, A1 is 0. 
corrAng(A,X,Y,A1):- Y==0, X@<0, A1 is 3.1415926535.
corrAng(A,X,Y,A1):- X==0, Y@>0, A1 is (3.1415926535/2).
corrAng(A,X,Y,A1):- X==0, Y@<0, A1 is 3*(3.1415926535/2).
corrAng(A,X,Y,A1):-  X@>=0, Y@>=0, A1 is A.
corrAng(A,X,Y,A1):-  X@>=0, Y@<0, A1 is (A+(2*3.1415926535)).
corrAng(A,X,Y,A1):-  X@<0, Y@>=0, A1 is (A+3.1415926535).
corrAng(A,X,Y,A1):-  X@<0, Y@<0, A1 is (A+3.1415926535).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MODALITA' FUGA

% Dopo aver raccolto l'obiettivo ruota di 180 gradi
nextCmd([cmd(pick,ARG)|PSTOR],_,_,fuga,CMD,PAR,PSTOR,STATE):- 
	CMD is turn, PAR is 180, STATE is fuga.
% Se la lista dei comandi è vuota, il ladro ha vinto
nextCmd([],_,_,fuga,victory,0,[],fuga).
% Se lista dei comandi non è vuota, cerca il successivo comando da eseguire
nextCmd(LIST,_,_,fuga,CMD,PAR,STOR,STATE):- STATE is fuga, findCommand(LIST,CMD,PAR,STOR).

% Se la lista dei comandi è vuota, il ladro ha vinto
findCommand([],victory,0,[]).
% Se era stato eseguito un comando TURN, lo riesegue con un angolo opposto
findCommand([cmd(turn,ARG)|PSTOR],turn,ARG1,PSTOR):- ARG1 is ARG*(-1).
% Se era stato eseguito un comando di walk, lo riesegue
findCommand([cmd(walk,ARG)|PSTOR],walk,ARG,PSTOR). 
% Se era stato eseguito un qualunque altro comando, lo scarta 
findCommand([_|PSTOR],CMD,PAR,STOR):- findCommand(PSTOR,CMD,PAR,STOR).
	
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	FUNZIONI GENERALI

% union(+M1,+M2,-Mout), aggiunge M2 in coda ad M1
union([],M1,M1).
union([M|Ms],M1,[M|Mo]):-union(Ms,M1,Mo).

% member(?E,?M,?O), trova tutte le occorrenze di E in M e restituisce il resto in o
member(E,[E|T],T).
member(E,[H|T],[H|R]):-member(E,T,R).

% first(?E,+M,-O), trova la prima occorrenza di E in M e restituisce il resto in O
first(E,[E|T],T):-!.
first(E,[H|T],[H|R]):-first(E,T,R).